from __future__ import annotations

import logging
from datetime import datetime, timedelta
from typing import Any

from mneia.core.llm import LLMClient
from mneia.memory.store import MemoryStore, StoredDocument

logger = logging.getLogger(__name__)

SUMMARY_SYSTEM = """You are a personal knowledge summarizer.
Given a collection of documents, produce a concise summary that captures:
- Key people mentioned and their roles/context
- Important decisions or action items
- Main topics and themes
- Notable beliefs or opinions expressed
Be direct and factual. Use bullet points. Keep under 500 words."""

TOPIC_SUMMARY_SYSTEM = """You summarize documents about a specific topic.
Focus on: key points, decisions made, open questions, and action items.
Be direct, use bullet points, keep under 300 words."""

PERSON_SUMMARY_SYSTEM = """You summarize what is known about a specific person from documents.
Focus on: their role, relationship to the user, recent interactions, key discussions.
Be direct, use bullet points, keep under 200 words."""


async def summarize_documents(
    docs: list[StoredDocument],
    llm: LLMClient,
    system: str = SUMMARY_SYSTEM,
) -> str:
    if not docs:
        return ""

    doc_texts = []
    for doc in docs[:20]:
        snippet = doc.content[:1000]
        doc_texts.append(f"[{doc.title} — {doc.source}, {doc.timestamp[:10]}]\n{snippet}")

    combined = "\n\n---\n\n".join(doc_texts)
    prompt = f"Summarize these {len(docs)} documents:\n\n{combined}"

    try:
        return await llm.generate(prompt, system=system)
    except Exception:
        logger.exception("Summarization failed")
        return ""


async def generate_daily_summary(
    store: MemoryStore,
    llm: LLMClient,
    date: datetime | None = None,
) -> str:
    if date is None:
        date = datetime.now()

    start = date.replace(hour=0, minute=0, second=0, microsecond=0)
    end = start + timedelta(days=1)

    docs = await store.get_documents_in_range(start.isoformat(), end.isoformat())
    if not docs:
        return ""

    return await summarize_documents(docs, llm, system=SUMMARY_SYSTEM)


async def generate_topic_summary(
    store: MemoryStore,
    llm: LLMClient,
    topic: str,
) -> str:
    docs = await store.search(topic, limit=15)
    if not docs:
        return ""

    return await summarize_documents(docs, llm, system=TOPIC_SUMMARY_SYSTEM)


async def generate_person_summary(
    store: MemoryStore,
    llm: LLMClient,
    person_name: str,
) -> str:
    docs = await store.search(person_name, limit=15)
    if not docs:
        return ""

    return await summarize_documents(docs, llm, system=PERSON_SUMMARY_SYSTEM)


async def generate_all_summaries(
    store: MemoryStore,
    llm: LLMClient,
    graph: Any,
    max_people: int = 10,
    max_topics: int = 8,
    on_progress: Any | None = None,
) -> dict[str, str]:
    summaries: dict[str, str] = {}

    def _progress(msg: str) -> None:
        if on_progress:
            on_progress(msg)

    recent = await store.get_recent(limit=30)
    if recent:
        _progress("Generating overview summary...")
        summaries["overview"] = await summarize_documents(recent, llm)

    people_nodes = [
        (nid, data) for nid, data in graph._graph.nodes(data=True)
        if data.get("entity_type") == "person"
    ]
    for i, (_nid, data) in enumerate(people_nodes[:max_people]):
        name = data.get("name", "")
        if name:
            _progress(f"Summarising person {i + 1}/{min(len(people_nodes), max_people)}: {name}")
            summary = await generate_person_summary(store, llm, name)
            if summary:
                summaries[f"person:{name}"] = summary

    topic_nodes = [
        (nid, data) for nid, data in graph._graph.nodes(data=True)
        if data.get("entity_type") in ("topic", "project")
    ]
    for i, (_nid, data) in enumerate(topic_nodes[:max_topics]):
        name = data.get("name", "")
        if name:
            _progress(f"Summarising topic {i + 1}/{min(len(topic_nodes), max_topics)}: {name}")
            summary = await generate_topic_summary(store, llm, name)
            if summary:
                summaries[f"topic:{name}"] = summary

    return summaries



from __future__ import annotations

import asyncio
import sys
from pathlib import Path
from typing import Any

import typer
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from mneia import __version__
from mneia.config import MNEIA_DIR, PID_PATH, MneiaConfig, ensure_dirs
from mneia.output import EXIT_INTERNAL_ERROR, get_output

app = typer.Typer(
    name="mneia",
    help="Autonomous multi-agent personal knowledge system",
    no_args_is_help=False,
    invoke_without_command=True,
    rich_markup_mode="rich",
)


def _version_callback(value: bool) -> None:
    if value:
        output = get_output()
        output.print(f"mneia v{__version__}")
        raise typer.Exit()


class _LazyConsole:
    """Proxy that delegates to the Output layer's Console.

    This ensures all existing console.print() calls respect NO_COLOR,
    --no-color, and other output configuration without changing 178 call sites.
    """

    def __getattr__(self, name: str) -> object:
        return getattr(get_output().console, name)


console = _LazyConsole()

BANNER = r"""
                         _
  _ __ ___  _ __   ___  (_) __ _
 | '_ ` _ \| '_ \ / _ \ | |/ _` |
 | | | | | | | | |  __/ | | (_| |
 |_| |_| |_|_| |_|\___| |_|\__,_|

"""


def _print_banner() -> None:
    output = get_output()
    output.print(Text(BANNER, style="bold cyan"))
    output.print(f"  v{__version__} — your personal knowledge agent\n", style="dim")


# --- Main callback: no args = interactive mode ---


@app.callback(invoke_without_command=True)
def main_callback(
    ctx: typer.Context,
    version: bool | None = typer.Option(
        None, "--version", callback=_version_callback,
        is_eager=True, help="Show version and exit",
    ),
    json_output: bool = typer.Option(
        False, "--json", "-j", help="Output as JSON",
    ),
    verbose: bool = typer.Option(
        False, "--verbose", help="Show debug output",
    ),
    quiet: bool = typer.Option(
        False, "--quiet", "-q", help="Suppress non-error output",
    ),
    no_input: bool = typer.Option(
        False, "--no-input", help="Disable interactive prompts",
    ),
    no_color: bool = typer.Option(
        False, "--no-color", help="Disable colored output",
    ),
) -> None:
    """Autonomous multi-agent personal knowledge system.

    Run with no arguments to enter interactive mode.
    Run with a command (e.g. mneia start) for direct execution.
    """
    output = get_output()
    output.configure(
        json_mode=json_output,
        verbose=verbose,
        quiet=quiet,
        no_input=no_input,
        no_color=no_color,
    )

    if ctx.invoked_subcommand is None:
        from mneia.interactive import run_interactive

        run_interactive()


# --- Top-level commands ---


@app.command()
def version() -> None:
    """Show mneia version."""
    output = get_output()
    output.emit(
        data={"version": __version__},
        rich_fn=lambda: console.print(f"mneia v{__version__}"),
    )


@app.command()
def start(
    detach: bool = typer.Option(False, "--detach", "-d", help="Run in background"),
    connectors: str | None = typer.Option(
        None, "--connectors", "-c", help="Comma-separated connector names"
    ),
    all_agents: bool = typer.Option(False, "--all", "-a", help="Start all agents"),
) -> None:
    """Start the mneia knowledge daemon."""
    import subprocess
    import time

    ensure_dirs()
    config = MneiaConfig.load()
    connector_filter = connectors.split(",") if connectors else None

    if detach:
        import os

        from mneia.config import MNEIA_DIR, PID_PATH, SOCKET_PATH

        if SOCKET_PATH.exists():
            pid_alive = False
            if PID_PATH.exists():
                try:
                    pid = int(PID_PATH.read_text().strip())
                    os.kill(pid, 0)
                    pid_alive = True
                except (ValueError, OSError):
                    pass
            if pid_alive:
                console.print("[yellow]Daemon already running.[/yellow]")
                return
            SOCKET_PATH.unlink(missing_ok=True)
            PID_PATH.unlink(missing_ok=True)

        python = sys.executable
        filter_arg = ""
        if connector_filter:
            filter_arg = f", connector_filter={connector_filter!r}"
        cmd = [
            python, "-c",
            "import asyncio, os; "
            f"open({str(PID_PATH)!r}, 'w').write(str(os.getpid())); "
            "from mneia.config import MneiaConfig; "
            "from mneia.core.lifecycle import AgentManager; "
            "config = MneiaConfig.load(); "
            f"manager = AgentManager(config{filter_arg}); "
            "asyncio.run(manager.run())"
        ]

        log_path = MNEIA_DIR / "logs" / "daemon.log"
        log_path.parent.mkdir(parents=True, exist_ok=True)
        log_file = open(str(log_path), "a")
        devnull = open("/dev/null")

        proc = subprocess.Popen(
            cmd,
            stdin=devnull,
            stdout=log_file,
            stderr=log_file,
            start_new_session=True,
        )

        PID_PATH.write_text(str(proc.pid))

        time.sleep(1.5)

        if SOCKET_PATH.exists():
            console.print(
                f"[green]● Daemon started[/green] [dim](PID {proc.pid})[/dim]"
            )
            console.print(f"  [dim]Logs: {log_path}[/dim]")
            console.print("  [dim]Stop with: [cyan]mneia stop[/cyan][/dim]")
        else:
            console.print(
                "[yellow]Daemon starting... check [cyan]mneia status[/cyan].[/yellow]"
            )
        return

    _print_banner()
    from mneia.core.lifecycle import AgentManager

    manager = AgentManager(config, connector_filter=connector_filter)

    console.print("[green]Starting mneia...[/green]")
    console.print("[dim]Press Ctrl+C to stop  |  Run with -d to detach[/dim]\n")
    try:
        asyncio.run(manager.run())
    except KeyboardInterrupt:
        console.print("\n[yellow]Shutting down...[/yellow]")


@app.command()
def stop(
    agent_name: str | None = typer.Argument(None, help="Stop specific agent (or omit for daemon)"),
    all_agents: bool = typer.Option(False, "--all", "-a", help="Stop all agents"),
) -> None:
    """Stop the mneia daemon or specific agents."""
    from mneia.core.lifecycle import send_command

    if agent_name and not all_agents:
        name = f"listener-{agent_name}" if not agent_name.startswith("listener-") else agent_name
        try:
            result = asyncio.run(send_command("stop_agent", name=name))
            if result.get("ok"):
                console.print(f"[yellow]Stopped agent: {result['stopped']}[/yellow]")
            elif result.get("error"):
                console.print(f"[red]{result['error']}[/red]")
        except (ConnectionRefusedError, FileNotFoundError, OSError):
            console.print("[red]Daemon is not running.[/red]")
        return

    console.print("[yellow]Sending stop signal...[/yellow]")
    try:
        asyncio.run(send_command("stop"))
        console.print("[green]mneia stopped.[/green]")
    except (ConnectionRefusedError, FileNotFoundError, OSError):
        if PID_PATH.exists():
            import os
            import signal as sig

            try:
                pid = int(PID_PATH.read_text().strip())
                os.kill(pid, sig.SIGTERM)
                console.print(f"[green]Sent SIGTERM to PID {pid}.[/green]")
            except (ProcessLookupError, ValueError):
                console.print("[dim]Stale PID file removed.[/dim]")
            PID_PATH.unlink(missing_ok=True)
        else:
            console.print("[red]mneia is not running.[/red]")


@app.command()
def status() -> None:
    """Show daemon status, agent states, and queue depth."""
    from mneia.core.lifecycle import send_command

    output = get_output()
    try:
        result = asyncio.run(send_command("status"))
        if result.get("running"):
            if output.is_json:
                output.json_result(result)
                return
            console.print(Panel("[green]mneia is running[/green]", title="Status"))
            table = Table(title="Agents")
            table.add_column("Agent", style="cyan")
            table.add_column("State", style="green")
            table.add_column("Docs Processed", justify="right")
            for agent in result.get("agents", []):
                table.add_row(
                    agent["name"], agent["state"],
                    str(agent.get("docs", 0)),
                )
            console.print(table)
        else:
            if output.is_json:
                output.json_result({"running": False})
                return
            console.print("[yellow]mneia is not running.[/yellow]")
    except (ConnectionRefusedError, FileNotFoundError, OSError):
        from mneia.config import PID_PATH, SOCKET_PATH

        SOCKET_PATH.unlink(missing_ok=True)
        PID_PATH.unlink(missing_ok=True)
        if output.is_json:
            output.json_result({"running": False})
            return
        console.print("[yellow]mneia is not running.[/yellow]")
        console.print("[dim]Start with: [cyan]mneia start -d[/cyan][/dim]")


# --- Config commands ---

config_app = typer.Typer(help="Manage mneia configuration")
app.add_typer(config_app, name="config")


@config_app.command("show")
def config_show() -> None:
    """Show current configuration."""
    config = MneiaConfig.load()
    console.print_json(config.model_dump_json(indent=2))


@config_app.command("set")
def config_set(key: str, value: str) -> None:
    """Set a configuration value (dot-separated key path)."""
    config = MneiaConfig.load()
    try:
        config.set_value(key, value)
        console.print(f"[green]Set {key} = {value}[/green]")
    except (AttributeError, KeyError) as e:
        console.print(f"[red]Invalid key: {key} ({e})[/red]")
        raise typer.Exit(1)


@config_app.command("setup")
def config_setup() -> None:
    """Interactive setup wizard."""
    ensure_dirs()
    _print_banner()
    config = MneiaConfig.load()

    console.print(Panel("Welcome to mneia setup!", title="Setup Wizard"))
    _llm_setup_wizard(config)

    context_dir = typer.prompt(
        "Context output directory",
        default=config.context_output_dir,
    )
    config.context_output_dir = context_dir

    config.save()
    console.print("\n[green]Configuration saved![/green]")
    _show_next_steps(config)


@config_app.command("llm")
def config_llm() -> None:
    """Configure LLM provider, API keys, and model selection."""
    ensure_dirs()
    config = MneiaConfig.load()

    console.print(Panel("LLM Configuration", title="Setup"))
    _llm_setup_wizard(config)
    config.save()
    console.print("\n[green]LLM configuration saved![/green]")
    console.print(f"  [dim]Provider:[/dim] [cyan]{config.llm.provider}[/cyan]")
    console.print(f"  [dim]Model:[/dim] [cyan]{config.llm.model}[/cyan]")


def _llm_setup_wizard(config: MneiaConfig) -> None:
    from mneia.core.llm_setup import (
        EMBEDDING_MODELS,
        PROVIDER_DISPLAY,
        get_models_for_provider,
    )

    console.print("\n[bold]Choose your LLM provider:[/bold]")
    providers = list(PROVIDER_DISPLAY.items())
    for i, (key, display) in enumerate(providers, 1):
        current = " [green](current)[/green]" if key == config.llm.provider else ""
        console.print(f"  [{i}] {display}{current}")

    choice = typer.prompt(
        "\nProvider number",
        default=str(next(
            i for i, (k, _) in enumerate(providers, 1)
            if k == config.llm.provider
        )),
    )
    try:
        provider_key = providers[int(choice) - 1][0]
    except (ValueError, IndexError):
        console.print("[red]Invalid choice, keeping current provider.[/red]")
        return

    config.llm.provider = provider_key

    if provider_key == "ollama":
        url = typer.prompt("Ollama base URL", default=config.llm.ollama_base_url)
        config.llm.ollama_base_url = url
        models = get_models_for_provider("ollama", url)
        if models:
            console.print("\n[bold]Available Ollama models:[/bold]")
            for i, m in enumerate(models, 1):
                current = " [green](current)[/green]" if m == config.llm.model else ""
                console.print(f"  [{i}] {m}{current}")
            model_choice = typer.prompt(
                "Model number or name", default=config.llm.model,
            )
            if model_choice.isdigit() and 1 <= int(model_choice) <= len(models):
                config.llm.model = models[int(model_choice) - 1]
            else:
                config.llm.model = model_choice
        else:
            console.print("[yellow]Ollama not reachable. Enter model name manually.[/yellow]")
            config.llm.model = typer.prompt("Model name", default=config.llm.model)
        config.llm.embedding_model = typer.prompt(
            "Embedding model", default=config.llm.embedding_model,
        )
    elif provider_key == "anthropic":
        if config.llm.anthropic_api_key:
            console.print(
                "[dim]Key already set. Enter new or press Enter to keep.[/dim]"
            )
        key = typer.prompt("Anthropic API key (sk-ant-...)", hide_input=True, default="")
        if key:
            config.llm.anthropic_api_key = key
        models = get_models_for_provider("anthropic")
        console.print("\n[bold]Available models:[/bold]")
        for i, m in enumerate(models, 1):
            console.print(f"  [{i}] {m}")
        model_choice = typer.prompt("Model number", default="1")
        if model_choice.isdigit() and 1 <= int(model_choice) <= len(models):
            config.llm.model = models[int(model_choice) - 1]
        config.llm.embedding_model = EMBEDDING_MODELS.get("anthropic", config.llm.embedding_model)
    elif provider_key == "openai":
        if config.llm.openai_api_key:
            console.print(
                "[dim]Key already set. Enter new or press Enter to keep.[/dim]"
            )
        key = typer.prompt("OpenAI API key (sk-...)", hide_input=True, default="")
        if key:
            config.llm.openai_api_key = key
        api_key = config.llm.openai_api_key or ""
        console.print("[dim]Fetching available models...[/dim]")
        models = get_models_for_provider("openai", api_key=api_key)
        console.print("\n[bold]Available models:[/bold]")
        for i, m in enumerate(models, 1):
            console.print(f"  [{i}] {m}")
        console.print(f"\n[dim]{len(models)} models available[/dim]")
        model_choice = typer.prompt("Model number or name", default="1")
        if model_choice.isdigit() and 1 <= int(model_choice) <= len(models):
            config.llm.model = models[int(model_choice) - 1]
        elif model_choice in models:
            config.llm.model = model_choice
        else:
            config.llm.model = model_choice
        config.llm.embedding_model = EMBEDDING_MODELS.get("openai", config.llm.embedding_model)
    elif provider_key == "google":
        if config.llm.google_api_key:
            console.print(
                "[dim]Key already set. Enter new or press Enter to keep.[/dim]"
            )
        key = typer.prompt("Google API key", hide_input=True, default="")
        if key:
            config.llm.google_api_key = key
        models = get_models_for_provider("google")
        console.print("\n[bold]Available models:[/bold]")
        for i, m in enumerate(models, 1):
            console.print(f"  [{i}] {m}")
        model_choice = typer.prompt("Model number", default="1")
        if model_choice.isdigit() and 1 <= int(model_choice) <= len(models):
            config.llm.model = models[int(model_choice) - 1]
        config.llm.embedding_model = EMBEDDING_MODELS.get("google", config.llm.embedding_model)

    console.print(
        f"\n[green]Selected:[/green] [cyan]{config.llm.provider}[/cyan] / "
        f"[cyan]{config.llm.model}[/cyan]"
    )


def _show_next_steps(config: MneiaConfig) -> None:
    enabled = [n for n, c in config.connectors.items() if c.enabled]
    console.print("\n[bold]Next steps:[/bold]")
    if not enabled:
        console.print("  1. Enable a connector: [cyan]mneia connector enable <name>[/cyan]")
        console.print("  2. Set it up: [cyan]mneia connector setup <name>[/cyan]")
        console.print("  3. Start the daemon: [cyan]mneia start -d[/cyan]")
    else:
        console.print(f"  [green]Connectors enabled:[/green] {', '.join(enabled)}")
        console.print("  1. Start the daemon: [cyan]mneia start -d[/cyan]")
        console.print("  2. Check status: [cyan]mneia status[/cyan]")
        console.print("  3. Ask a question: [cyan]mneia ask 'what do I know?'[/cyan]")
    console.print("  [dim]See all connectors: mneia connector list[/dim]")
    console.print("  [dim]Interactive mode: just run mneia[/dim]")


@config_app.command("reset")
def config_reset() -> None:
    """Reset configuration to defaults."""
    if typer.confirm("Reset all configuration to defaults?"):
        config = MneiaConfig()
        config.save()
        console.print("[green]Configuration reset.[/green]")


# --- Connector commands ---

connector_app = typer.Typer(help="Manage data connectors")
app.add_typer(connector_app, name="connector")


@connector_app.command("list")
def connector_list() -> None:
    """List available connectors and their status."""
    config = MneiaConfig.load()
    from mneia.connectors import get_available_connectors

    output = get_output()
    available = get_available_connectors()

    if output.is_json:
        data = []
        for m in available:
            cc = config.connectors.get(m.name)
            data.append({
                "name": m.name,
                "display_name": m.display_name,
                "enabled": bool(cc and cc.enabled),
                "auth_type": m.auth_type,
            })
        output.json_result({"connectors": data})
        return

    table = Table(title="Connectors")
    table.add_column("Name", style="cyan")
    table.add_column("Description")
    table.add_column("Status", style="green")
    table.add_column("Auth")

    for manifest in available:
        conn_config = config.connectors.get(manifest.name)
        enabled = conn_config and conn_config.enabled
        status_text = "[green]enabled[/green]" if enabled else "[dim]disabled[/dim]"
        table.add_row(
            manifest.name, manifest.display_name,
            status_text, manifest.auth_type,
        )

    console.print(table)


@connector_app.command("enable")
def connector_enable(name: str) -> None:
    """Enable a connector."""
    config = MneiaConfig.load()
    from mneia.connectors import get_connector_manifest

    manifest = get_connector_manifest(name)
    if not manifest:
        console.print(f"[red]Unknown connector: {name}[/red]")
        raise typer.Exit(1)

    if name not in config.connectors:
        from mneia.config import ConnectorConfig

        config.connectors[name] = ConnectorConfig(
            enabled=True,
            poll_interval_seconds=manifest.poll_interval_seconds,
        )
    else:
        config.connectors[name].enabled = True

    config.save()
    console.print(f"[green]Enabled connector: {name}[/green]")
    console.print(f"Run [cyan]mneia connector setup {name}[/cyan] to configure it.")


@connector_app.command("disable")
def connector_disable(name: str) -> None:
    """Disable a connector."""
    config = MneiaConfig.load()
    if name in config.connectors:
        config.connectors[name].enabled = False
        config.save()
        console.print(f"[yellow]Disabled connector: {name}[/yellow]")
    else:
        console.print(f"[red]Connector not configured: {name}[/red]")


@connector_app.command("setup")
def connector_setup(name: str) -> None:
    """Run interactive setup for a connector with guided instructions."""
    config = MneiaConfig.load()
    from mneia.connectors import create_connector
    from mneia.core.llm_setup import get_connector_help

    connector = create_connector(name)
    if not connector:
        console.print(f"[red]Unknown connector: {name}[/red]")
        raise typer.Exit(1)

    help_info = get_connector_help(name)
    manifest = connector.manifest

    console.print(Panel(
        f"Setting up [cyan]{manifest.display_name}[/cyan]",
        title="Connector Setup",
    ))

    if help_info:
        console.print(f"\n  [bold]{manifest.display_name}[/bold]")
        console.print(f"  [dim]{help_info['description']}[/dim]\n")
        console.print("  [bold]Prerequisites:[/bold]")
        for line in help_info["prerequisites"].split("\n"):
            console.print(f"    {line}")
        console.print("\n  [bold]What you'll need:[/bold]")
        console.print(f"    {help_info['setup_help']}\n")
    else:
        console.print(f"\n  [dim]{manifest.description}[/dim]")
        console.print(f"  [dim]Auth: {manifest.auth_type}[/dim]\n")
        if manifest.required_config:
            console.print(f"  [bold]Required:[/bold] {', '.join(manifest.required_config)}")
        if manifest.optional_config:
            console.print(f"  [dim]Optional: {', '.join(manifest.optional_config)}[/dim]\n")

    existing = config.connectors.get(name)
    if existing and existing.enabled:
        console.print(f"\n[green]{name}[/green] is already configured.")
        reconfigure = typer.confirm("  Reconfigure? (removes previous settings)", default=False)
        if not reconfigure:
            console.print("[dim]Keeping existing configuration.[/dim]")
            return
        console.print("[dim]Removing previous configuration...[/dim]")
        del config.connectors[name]
        config.save()

    if name not in config.connectors:
        from mneia.config import ConnectorConfig

        config.connectors[name] = ConnectorConfig(enabled=True)

    settings = connector.interactive_setup()
    config.connectors[name].settings = settings
    config.connectors[name].enabled = True
    config.save()

    console.print(f"\n[green]{manifest.display_name} configured and enabled![/green]")

    if help_info:
        console.print("\n[bold]Next steps:[/bold]")
        for line in help_info["next_steps"].split("\n"):
            console.print(f"  {line}")
    else:
        console.print("\nNext: [cyan]mneia start -d[/cyan] to begin syncing.")
    console.print()


@connector_app.command("sync")
def connector_sync(name: str) -> None:
    """Trigger immediate sync for a connector."""
    config = MneiaConfig.load()
    conn_config = config.connectors.get(name)

    # Resolve account-based names (e.g. "google-drive" → "google-drive-personal")
    if not conn_config or not conn_config.enabled:
        for key, cc in config.connectors.items():
            if key.startswith(f"{name}-") and cc.enabled:
                name = key
                conn_config = cc
                break

    if not conn_config or not conn_config.enabled:
        console.print(f"[red]Connector {name} is not enabled.[/red]")
        raise typer.Exit(1)

    from mneia.connectors import create_connector
    from mneia.pipeline.ingest import ingest_connector

    connector = create_connector(name)
    if not connector:
        console.print(f"[red]Unknown connector: {name}[/red]")
        raise typer.Exit(1)

    console.print(f"[cyan]Syncing {name}...[/cyan]")
    result = asyncio.run(ingest_connector(connector, conn_config, config))
    console.print(f"[green]Synced {result.documents_ingested} documents from {name}[/green]")

    if conn_config.last_checkpoint != result.checkpoint:
        conn_config.last_checkpoint = result.checkpoint
        config.save()


@connector_app.command("start-agent")
def connector_start_agent(name: str) -> None:
    """Start a connector's listener agent (daemon must be running)."""
    from mneia.core.lifecycle import send_command

    agent_name = f"listener-{name}" if not name.startswith("listener-") else name
    try:
        result = asyncio.run(send_command("start_agent", name=agent_name))
        if result.get("ok"):
            console.print(f"[green]Started agent: {result['started']}[/green]")
        elif result.get("error"):
            console.print(f"[red]{result['error']}[/red]")
    except (ConnectionRefusedError, FileNotFoundError, OSError):
        console.print("[red]Daemon is not running. Start with: mneia start[/red]")


@connector_app.command("stop-agent")
def connector_stop_agent(name: str) -> None:
    """Stop a connector's listener agent."""
    from mneia.core.lifecycle import send_command

    agent_name = f"listener-{name}" if not name.startswith("listener-") else name
    try:
        result = asyncio.run(send_command("stop_agent", name=agent_name))
        if result.get("ok"):
            console.print(f"[yellow]Stopped agent: {result['stopped']}[/yellow]")
        elif result.get("error"):
            console.print(f"[red]{result['error']}[/red]")
    except (ConnectionRefusedError, FileNotFoundError, OSError):
        console.print("[red]Daemon is not running.[/red]")


@connector_app.command("agents")
def connector_agents() -> None:
    """List running connector agents."""
    from mneia.core.lifecycle import send_command

    try:
        result = asyncio.run(send_command("list_agents"))
        agents = result.get("agents", [])
        if not agents:
            console.print("[dim]No agents running.[/dim]")
            return

        table = Table(title="Running Agents")
        table.add_column("Name", style="cyan")
        table.add_column("State", style="green")
        for a in agents:
            table.add_row(a["name"], a["state"])
        console.print(table)
    except (ConnectionRefusedError, FileNotFoundError, OSError):
        console.print("[red]Daemon is not running.[/red]")


# --- Memory commands ---

memory_app = typer.Typer(help="Browse and search your knowledge")
app.add_typer(memory_app, name="memory")


@memory_app.command("stats")
def memory_stats() -> None:
    """Show memory statistics."""
    from mneia.memory.store import MemoryStore

    output = get_output()
    store = MemoryStore()
    stats = asyncio.run(store.get_stats())

    if output.is_json:
        output.json_result(stats)
        return

    table = Table(title="Memory Statistics")
    table.add_column("Metric", style="cyan")
    table.add_column("Value", justify="right", style="green")

    table.add_row("Total documents", str(stats.get("total_documents", 0)))
    table.add_row("Total entities", str(stats.get("total_entities", 0)))
    table.add_row("Total associations", str(stats.get("total_associations", 0)))

    if "by_source" in stats:
        table.add_section()
        for source, count in stats["by_source"].items():
            table.add_row(f"  {source}", str(count))

    console.print(table)


@memory_app.command("search")
def memory_search(
    query: str,
    limit: int = typer.Option(10, "--limit", "-n", help="Max results"),
) -> None:
    """Full-text search across all stored knowledge."""
    from mneia.memory.store import MemoryStore

    output = get_output()
    store = MemoryStore()
    results = asyncio.run(store.search(query, limit=limit))

    if output.is_json:
        data = [
            {
                "title": doc.title,
                "source": doc.source,
                "content": doc.content[:500],
                "timestamp": doc.timestamp,
            }
            for doc in results
        ]
        output.json_result({"query": query, "results": data})
        return

    if not results:
        console.print("[yellow]No results found.[/yellow]")
        return

    for doc in results:
        panel_content = doc.content[:500]
        if len(doc.content) > 500:
            panel_content += "..."
        console.print(
            Panel(
                panel_content,
                title=f"[cyan]{doc.title}[/cyan] [dim]({doc.source})[/dim]",
                subtitle=f"[dim]{doc.timestamp}[/dim]",
            )
        )


@memory_app.command("recent")
def memory_recent(
    limit: int = typer.Option(10, "--limit", "-n", help="Max results"),
) -> None:
    """Show recently ingested items."""
    from mneia.memory.store import MemoryStore

    store = MemoryStore()
    results = asyncio.run(store.get_recent(limit=limit))

    if not results:
        console.print("[yellow]No documents stored yet.[/yellow]")
        return

    table = Table(title="Recent Documents")
    table.add_column("Title", style="cyan")
    table.add_column("Source")
    table.add_column("Type")
    table.add_column("Timestamp", style="dim")

    for doc in results:
        table.add_row(doc.title[:60], doc.source, doc.content_type, str(doc.timestamp))

    console.print(table)


@memory_app.command("purge")
def memory_purge(
    source: str | None = typer.Option(None, "--source", "-s", help="Purge only this source"),
    confirm: bool = typer.Option(False, "--confirm", help="Skip confirmation"),
) -> None:
    """Clear stored memory."""
    if not confirm:
        msg = f"Purge all data from {source}?" if source else "Purge ALL stored data?"
        if not typer.confirm(msg):
            raise typer.Abort()

    from mneia.memory.store import MemoryStore

    store = MemoryStore()
    asyncio.run(store.purge(source=source))
    console.print("[green]Memory purged.[/green]")


# --- Extract command ---


@app.command()
def extract(
    limit: int = typer.Option(50, "--limit", "-n", help="Max documents to process"),
) -> None:
    """Run entity extraction on unprocessed documents."""
    from mneia.core.llm import LLMClient
    from mneia.memory.graph import KnowledgeGraph
    from mneia.memory.store import MemoryStore
    from mneia.pipeline.extract import extract_and_store

    config = MneiaConfig.load()
    store = MemoryStore()
    graph = KnowledgeGraph()
    llm = LLMClient(config.llm)

    async def _run() -> None:
        docs = await store.get_unprocessed(limit=limit)
        if not docs:
            console.print("[dim]No unprocessed documents.[/dim]")
            return

        console.print(f"[cyan]Extracting entities from {len(docs)} documents...[/cyan]")
        total_entities = 0
        total_rels = 0

        for i, doc in enumerate(docs, 1):
            console.print(f"  [{i}/{len(docs)}] {doc.title[:60]}...", end="")
            try:
                result = await extract_and_store(doc, llm, store, graph)
                total_entities += result["entities"]
                total_rels += result["relationships"]
                console.print(f" [green]✓[/green] {result['entities']}E {result['relationships']}R")
            except Exception as e:
                console.print(f" [red]✗ {e}[/red]")

        console.print(f"\n[green]Extracted {total_entities} entities, {total_rels} relationships[/green]")
        stats = graph.get_stats()
        console.print(f"[dim]Graph now has {stats['total_nodes']} entities, {stats['total_edges']} relationships[/dim]")

    try:
        asyncio.run(_run())
    finally:
        asyncio.run(llm.close())


# --- Context commands ---

context_app = typer.Typer(help="Manage generated context files")
app.add_typer(context_app, name="context")


@context_app.command("generate")
def context_generate() -> None:
    """Force regenerate all context .md files."""
    from mneia.core.llm import LLMClient
    from mneia.memory.graph import KnowledgeGraph
    from mneia.memory.store import MemoryStore
    from mneia.pipeline.generate import generate_context_files

    config = MneiaConfig.load()
    store = MemoryStore()
    graph = KnowledgeGraph()
    llm = LLMClient(config.llm)

    console.print("[cyan]Generating context files...[/cyan]")
    try:
        generated = asyncio.run(generate_context_files(config, store, graph, llm))
        if generated:
            for name in generated:
                console.print(f"  [green]✓[/green] {name}")
            console.print(f"\n[green]Generated {len(generated)} context file(s) in {config.context_output_dir}[/green]")
        else:
            console.print("[yellow]No files generated. Check templates and data.[/yellow]")
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
    finally:
        asyncio.run(llm.close())


@context_app.command("generate-claude")
def context_generate_claude(
    output: str = typer.Option(
        "", "--output", "-o",
        help="Output path (default: ~/.mneia/claude-context.md)",
    ),
) -> None:
    """Write a Claude Code context file summarising your knowledge base.

    The file is read by the mneia Claude Code skill automatically, giving
    Claude personalised context about your people, projects, and recent activity.
    """
    from mneia.memory.graph import KnowledgeGraph
    from mneia.memory.store import MemoryStore

    out_path = Path(output) if output else Path.home() / ".mneia" / "claude-context.md"
    out_path.parent.mkdir(parents=True, exist_ok=True)

    config = MneiaConfig.load()
    store = MemoryStore()
    graph = KnowledgeGraph()

    console.print("[cyan]Building Claude Code context...[/cyan]")

    try:
        lines: list[str] = []
        lines.append("# mneia — Personal Knowledge Context")
        lines.append(f"*Generated: {__import__('datetime').datetime.now().strftime('%Y-%m-%d %H:%M')}*\n")

        # Doc counts per source
        stats = asyncio.run(store.get_stats()) if hasattr(store, "get_stats") else {}
        by_source: dict[str, int] = stats.get("by_source", {}) if isinstance(stats, dict) else {}
        if by_source:
            lines.append("## Knowledge Base")
            lines.append(f"- **Total**: {stats.get('total_documents', 0):,} documents")
            for src, count in sorted(by_source.items(), key=lambda x: -x[1]):
                lines.append(f"  - {src}: {count:,}")
            lines.append("")

        # Graph summary
        graph_stats = graph.get_stats()
        if graph_stats.get("total_nodes", 0) > 0:
            lines.append("## Knowledge Graph")
            lines.append(
                f"- **{graph_stats['total_nodes']} entities**, "
                f"**{graph_stats['total_edges']} relationships**"
            )
            # List entity types
            type_counts: dict[str, int] = {}
            for _nid, data in graph._graph.nodes(data=True):
                etype = data.get("entity_type", "unknown")
                type_counts[etype] = type_counts.get(etype, 0) + 1
            for etype, cnt in sorted(type_counts.items(), key=lambda x: -x[1])[:8]:
                lines.append(f"  - {etype}: {cnt}")
            lines.append("")

            # Notable people entities with their properties — filter noise
            import re as _re
            def _is_real_person(name: str) -> bool:
                if not name or len(name) < 4 or len(name) > 60:
                    return False
                # Skip if starts with a digit
                if name[0].isdigit():
                    return False
                # Skip if mostly digits (phone numbers, IDs)
                if _re.search(r"\d{5,}", name):
                    return False
                # Skip if all-caps word (acronyms like "API", "PM", "AI")
                words = name.split()
                if all(w.isupper() or w.isdigit() for w in words):
                    return False
                # Skip generic role/job-title words — these are not people's names
                _role_words = {
                    "engineer", "strategist", "specialist", "analyst",
                    "manager", "leader", "professional", "member",
                    "user", "agent", "team", "group", "researcher",
                    "director", "architect", "advisor", "creator",
                    "buyer", "auditor", "operator", "producer",
                    "coach", "developer", "designer", "tester",
                    "writer", "reporter", "editor", "assistant",
                    "shepherd", "guardian", "commander", "officer",
                    "inspector", "integrator", "maintainer", "tracker",
                    "generator", "injector", "synthesizer", "responder",
                    "reporter", "collector", "distributor",
                }
                name_words_lower = {w.lower().rstrip("s") for w in name.split()}
                if name_words_lower & _role_words:
                    # Only keep if it looks exactly like First Lastname (two proper-case words)
                    if not _re.fullmatch(r"[A-Z][a-z]+(?:['\- ][A-Z][a-z]+)+", name):
                        return False
                # Must contain at least one word with mixed case (real name)
                if not _re.search(r"[A-Z][a-z]", name):
                    return False
                return True

            people_nodes = [
                (nid, data) for nid, data in graph._graph.nodes(data=True)
                if data.get("entity_type", "").lower() in ("person", "people")
                and _is_real_person(data.get("name", ""))
            ]
            if people_nodes:
                lines.append("## People in Your Knowledge Base")
                for nid, data in sorted(people_nodes, key=lambda x: x[1].get("name", ""))[:40]:
                    name = data.get("name", nid)
                    props = data.get("properties", {})
                    desc = props.get("description", "")
                    detail_parts = [f"**{name}**"]
                    if desc and len(desc) < 120:
                        detail_parts.append(desc)
                    for key in ("school", "role", "company", "relationship", "age"):
                        if props.get(key):
                            detail_parts.append(f"{key}: {props[key]}")
                    lines.append(f"- {' — '.join(detail_parts)}")
                lines.append("")

        # Recent docs (last 10)
        recent = asyncio.run(store.search("recent activity notes", limit=10))
        if recent:
            lines.append("## Recently Indexed")
            for doc in recent[:10]:
                ts = (
                    doc.timestamp.strftime("%Y-%m-%d")
                    if hasattr(doc.timestamp, "strftime") else str(doc.timestamp)[:10]
                )
                lines.append(f"- [{doc.source}] **{doc.title}** ({ts})")
            lines.append("")

        lines.append(
            "*This file is auto-generated by mneia. "
            "Run `mneia context generate-claude` to refresh.*"
        )

        out_path.write_text("\n".join(lines), encoding="utf-8")
        console.print(f"[green]✓ Written to {out_path}[/green]")
        console.print(
            f"[dim]Claude Code will load this automatically via the mneia skill.[/dim]"
        )

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        raise typer.Exit(1)


@context_app.command("show")
def context_show() -> None:
    """List generated context files."""
    config = MneiaConfig.load()
    output_dir = Path(config.context_output_dir)
    if not output_dir.exists():
        console.print("[yellow]No context files generated yet.[/yellow]")
        return

    table = Table(title="Context Files")
    table.add_column("File", style="cyan")
    table.add_column("Size", justify="right")
    table.add_column("Modified", style="dim")

    for f in sorted(output_dir.glob("*.md")):
        stat = f.stat()
        size = f"{stat.st_size:,} bytes"
        from datetime import datetime

        modified = datetime.fromtimestamp(stat.st_mtime).strftime("%Y-%m-%d %H:%M")
        table.add_row(f.name, size, modified)

    console.print(table)


@context_app.command("link")
def context_link(target_dir: str) -> None:
    """Symlink context files to a project directory."""
    config = MneiaConfig.load()
    source = Path(config.context_output_dir)
    target = Path(target_dir)

    if not source.exists():
        console.print("[red]No context files generated yet. Run mneia context generate first.[/red]")
        raise typer.Exit(1)

    target.mkdir(parents=True, exist_ok=True)
    for f in source.glob("*.md"):
        link = target / f.name
        if link.exists() or link.is_symlink():
            link.unlink()
        link.symlink_to(f)
        console.print(f"[green]Linked {f.name} -> {target}[/green]")


# --- Ask command ---


_SOURCE_COLORS: dict[str, str] = {
    "google-calendar": "bright_green",
    "gmail": "bright_blue",
    "google-drive": "bright_yellow",
    "granola": "bright_magenta",
    "obsidian": "bright_cyan",
    "web": "bright_white",
    "knowledge-agent": "bright_red",
    "autonomous-insight": "bright_red",
}


def _detect_source_hints(question: str) -> list[str] | None:
    q = question.lower()
    source_keywords: dict[str, list[str]] = {
        "google-calendar": [
            "calendar", "meeting", "meetings", "event", "events",
            "schedule", "appointment", "standup", "sync", "1:1",
            "one-on-one", "invite",
        ],
        "gmail": [
            "email", "emails", "mail", "inbox", "gmail",
            "message", "thread", "sent",
        ],
        "google-drive": [
            "drive", "doc", "docs", "sheet", "sheets",
            "slide", "slides", "google doc",
        ],
        "granola": [
            "meeting", "meeting notes", "transcript",
            "transcription", "conversation", "said",
        ],
        "local-folders": [
            "file", "files", "folder", "folders", "directory",
            "local", "document", "documents", "code", "script",
        ],
    }
    matched = []
    for source, keywords in source_keywords.items():
        if any(kw in q for kw in keywords):
            matched.append(source)
    return matched if matched else None


@app.command()
def ask(
    question: str,
    source: str | None = typer.Option(
        None, "--source", "-s", help="Limit to source",
    ),
) -> None:
    """Ask a question about your knowledge (single query with RAG)."""
    from mneia.conversation import ConversationEngine

    config = MneiaConfig.load()
    engine = ConversationEngine(config)

    async def _run() -> None:
        hints = _detect_source_hints(question) if not source else None
        result = await engine.ask(
            question,
            source_filter=source,
            source_hints=hints,
        )
        from rich.markdown import Markdown
        from rich.panel import Panel as RichPanel

        has_context = bool(result.citations)

        if has_context:
            sources = sorted({c.source for c in result.citations})
            tags = " ".join(
                f"[{_SOURCE_COLORS.get(s, 'dim')}]{s}"
                f"[/{_SOURCE_COLORS.get(s, 'dim')}]"
                for s in sources
            )
            console.print(
                f"\n  [green]\u25cf[/green] [dim]Context from:[/dim] "
                f"{tags} [dim]({len(result.citations)} docs)[/dim]"
            )
        else:
            console.print(
                "\n  [yellow]\u25cb[/yellow] "
                "[dim]No matching context \u2014 "
                "answering from general knowledge[/dim]"
            )

        console.print()
        border = "green" if has_context else "yellow"
        console.print(
            RichPanel(
                Markdown(result.answer),
                border_style=border,
                padding=(1, 2),
            )
        )

        if result.citations:
            console.print("[dim]Sources:[/dim]")
            for cite in result.citations:
                color = _SOURCE_COLORS.get(cite.source, "dim")
                console.print(
                    f"  [dim]-[/dim] [{color}]{cite.source}"
                    f"[/{color}] [dim]{cite.title}[/dim]"
                )

        if result.suggested_followups:
            console.print("\n[dim]Follow-up questions:[/dim]")
            for q in result.suggested_followups:
                console.print(f"  [cyan]- {q}[/cyan]")
        console.print()

        await engine.close()

    try:
        asyncio.run(_run())
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        console.print(
            "[dim]Ensure Ollama is running "
            "or an API key is configured.[/dim]"
        )


@app.command()
def chat() -> None:
    """Interactive multi-turn conversation about your knowledge."""
    from prompt_toolkit import PromptSession
    from prompt_toolkit.formatted_text import HTML
    from prompt_toolkit.history import FileHistory
    from rich.markdown import Markdown

    from mneia.conversation import ConversationEngine

    config = MneiaConfig.load()
    engine = ConversationEngine(config)

    console.print("[cyan]Entering chat mode. Type 'exit' or Ctrl+D to leave.[/cyan]")
    console.print("[dim]Your conversation history is preserved across questions.[/dim]\n")

    session: PromptSession[str] = PromptSession(
        history=FileHistory(str(MNEIA_DIR / "chat_history.txt")),
    )

    try:
        while True:
            try:
                question = session.prompt(
                    HTML("<ansibrightcyan><b>you</b></ansibrightcyan> <ansigray>›</ansigray> "),
                )
                question = question.strip()
                if not question:
                    continue
                if question.lower() in ("exit", "quit", "/exit", "/quit"):
                    break
                if question.lower() in ("clear", "/clear"):
                    engine.clear_history()
                    console.print("[dim]Conversation cleared.[/dim]\n")
                    continue

                with console.status("[dim italic]Thinking...[/dim italic]", spinner="dots"):
                    result = asyncio.run(engine.ask(question))

                console.print()
                console.print(Markdown(result.answer))

                if result.citations:
                    console.print("\n[dim]Sources:[/dim]")
                    for cite in result.citations:
                        console.print(f"  [dim]- {cite.title} ({cite.source})[/dim]")

                if result.suggested_followups:
                    console.print("\n[dim]You could also ask:[/dim]")
                    for q in result.suggested_followups:
                        console.print(f"  [cyan]- {q}[/cyan]")
                console.print()

            except KeyboardInterrupt:
                console.print()
                continue
            except EOFError:
                break
    finally:
        asyncio.run(engine.close())
        console.print("\n[dim]Chat ended.[/dim]")


# --- Graph commands ---

graph_app = typer.Typer(help="Explore your knowledge graph")
app.add_typer(graph_app, name="graph")


@graph_app.command("show")
def graph_show() -> None:
    """Show knowledge graph summary."""
    from mneia.memory.graph import KnowledgeGraph

    graph = KnowledgeGraph()
    stats = graph.get_stats()

    table = Table(title="Knowledge Graph")
    table.add_column("Metric", style="cyan")
    table.add_column("Value", justify="right", style="green")

    table.add_row("Total entities", str(stats["total_nodes"]))
    table.add_row("Total relationships", str(stats["total_edges"]))

    if stats.get("by_type"):
        table.add_section()
        for etype, count in sorted(stats["by_type"].items()):
            table.add_row(f"  {etype}", str(count))

    console.print(table)


@graph_app.command("entities")
def graph_entities(
    entity_type: str | None = typer.Option(None, "--type", "-t", help="Filter by type"),
) -> None:
    """List entities in the knowledge graph."""
    from mneia.memory.graph import KnowledgeGraph

    graph = KnowledgeGraph()

    table = Table(title="Entities")
    table.add_column("Name", style="cyan")
    table.add_column("Type", style="green")
    table.add_column("Description", style="dim")

    for nid, data in graph._graph.nodes(data=True):
        etype = data.get("entity_type", "unknown")
        if entity_type and etype != entity_type:
            continue
        name = data.get("name", nid)
        desc = data.get("properties", {}).get("description", "")
        table.add_row(name, etype, desc[:80])

    console.print(table)


def _find_graph_node(
    graph: Any, name: str, preferred_type: str | None = None,
) -> str | None:
    name_lower = name.lower()
    if preferred_type:
        candidate = f"{preferred_type}:{name_lower.replace(' ', '-')}"
        if candidate in graph._graph:
            return candidate
    for nid, data in graph._graph.nodes(data=True):
        if data.get("name", "").lower() == name_lower:
            return nid
    slug = name_lower.replace(" ", "-")
    for nid in graph._graph.nodes:
        if nid.endswith(f":{slug}"):
            return nid
    for nid, data in graph._graph.nodes(data=True):
        if name_lower in data.get("name", "").lower():
            return nid
    return None


def _show_graph_entity(name: str, preferred_type: str) -> None:
    from mneia.memory.graph import KnowledgeGraph

    graph = KnowledgeGraph()
    node_id = _find_graph_node(graph, name, preferred_type)
    if not node_id:
        console.print(f"[yellow]No entity found matching: {name}[/yellow]")
        similar = []
        for nid, data in graph._graph.nodes(data=True):
            n = data.get("name", "")
            if name.lower() in n.lower():
                etype = data.get("entity_type", "unknown")
                similar.append(f"{n} ({etype})")
        if similar:
            console.print("[dim]Similar entities:[/dim]")
            for s in similar[:5]:
                console.print(f"  [dim]- {s}[/dim]")
        return

    node_data = graph._graph.nodes[node_id]
    display_name = node_data.get("name", name)
    etype = node_data.get("entity_type", "unknown")
    label = f"[bold cyan]{display_name}[/bold cyan] [dim]({etype})[/dim]"
    console.print(Panel(label, title=etype.title()))

    props = node_data.get("properties", {})
    if props.get("description"):
        console.print(f"[dim]{props['description'][:200]}[/dim]")

    result = graph.get_neighbors(node_id, depth=2)
    if result["edges"]:
        table = Table(show_header=True)
        table.add_column("Relationship", style="green")
        table.add_column("Entity", style="cyan")
        for edge in result["edges"]:
            other = edge["target"] if edge["source"] == node_id else edge["source"]
            other_data = graph._graph.nodes.get(other, {})
            fallback = other.split(":", 1)[-1].replace("-", " ").title()
            other_name = other_data.get("name", fallback)
            table.add_row(edge["relation"], other_name)
        console.print(table)
    else:
        console.print("[dim]No relationships found.[/dim]")


@graph_app.command("person")
def graph_person(name: str) -> None:
    """Show everything known about a person."""
    _show_graph_entity(name, "person")


@graph_app.command("topic")
def graph_topic(name: str) -> None:
    """Show everything known about a topic."""
    _show_graph_entity(name, "topic")


@graph_app.command("export")
def graph_export(
    fmt: str = typer.Option("json", "--format", "-f", help="Export format (json)"),
) -> None:
    """Export the knowledge graph."""
    import json as json_mod

    from mneia.memory.graph import KnowledgeGraph

    graph = KnowledgeGraph()
    data = graph.export_json()
    console.print_json(json_mod.dumps(data, indent=2, default=str))


# --- Permission commands ---

permission_app = typer.Typer(help="Manage operation permissions")
app.add_typer(permission_app, name="permission")


@permission_app.command("grant")
def permission_grant(
    operation: str,
    ttl: int = typer.Option(24, "--ttl", help="Hours until expiry"),
) -> None:
    """Grant permission for a risky operation."""
    from mneia.core.permissions_db import PermissionsDB
    from mneia.core.safety import get_permission

    perm = get_permission(operation)
    if not perm:
        console.print(f"[red]Unknown operation: {operation}[/red]")
        from mneia.core.safety import list_permissions
        ops = [p.operation for p in list_permissions()]
        console.print(f"[dim]Available: {', '.join(ops)}[/dim]")
        raise typer.Exit(1)

    db = PermissionsDB()
    db.approve(operation, ttl_hours=ttl)
    console.print(
        f"[green]Granted '{operation}' for {ttl}h "
        f"(risk: {perm.risk_level.value})[/green]"
    )


@permission_app.command("revoke")
def permission_revoke(operation: str) -> None:
    """Revoke a previously granted permission."""
    from mneia.core.permissions_db import PermissionsDB

    db = PermissionsDB()
    db.revoke(operation)
    console.print(f"[yellow]Revoked: {operation}[/yellow]")


@permission_app.command("list")
def permission_list() -> None:
    """List all permissions and their approval status."""
    from mneia.core.permissions_db import PermissionsDB
    from mneia.core.safety import list_permissions

    db = PermissionsDB()
    approvals = {a["operation"]: a for a in db.list_approvals()}
    perms = list_permissions()

    table = Table(title="Permissions")
    table.add_column("Operation", style="cyan")
    table.add_column("Risk")
    table.add_column("Status")
    table.add_column("Expires")

    for perm in perms:
        risk_colors = {
            "low": "green",
            "medium": "yellow",
            "high": "red",
            "critical": "bold red",
        }
        color = risk_colors.get(perm.risk_level.value, "white")
        risk_text = f"[{color}]{perm.risk_level.value}[/{color}]"

        approval = approvals.get(perm.operation)
        if perm.risk_level.value == "low":
            status = "[green]auto-approved[/green]"
            expires = "-"
        elif approval:
            status = "[green]granted[/green]"
            exp = approval.get("expires_at", "")
            expires = exp[:19] if exp else "never"
        else:
            status = "[dim]not granted[/dim]"
            expires = "-"

        table.add_row(
            perm.operation, risk_text, status, expires,
        )

    console.print(table)


# --- MCP commands ---

mcp_app = typer.Typer(help="MCP server for AI tool integration")
app.add_typer(mcp_app, name="mcp")


@mcp_app.command("serve")
def mcp_serve() -> None:
    """Start the MCP server (stdio transport for Claude Code integration)."""
    try:
        from mneia.mcp.server import run_server
    except ImportError:
        console.print(
            "[red]MCP package not installed. "
            "Install with: pip install 'mneia[mcp]'[/red]"
        )
        raise typer.Exit(1)

    run_server()


# --- Marketplace commands ---

marketplace_app = typer.Typer(help="Browse and install connectors")
app.add_typer(marketplace_app, name="marketplace")


@marketplace_app.command("search")
def marketplace_search(query: str) -> None:
    """Search available connectors in the marketplace."""
    from mneia.marketplace.registry import search_index

    results = search_index(query)
    if not results:
        console.print(f"[yellow]No connectors found matching: {query}[/yellow]")
        return

    table = Table(title=f"Marketplace results for '{query}'")
    table.add_column("Name", style="cyan")
    table.add_column("Description")
    table.add_column("Author", style="dim")
    table.add_column("Version")
    table.add_column("Status")

    for entry in results:
        status = "[green]installed[/green]" if entry.installed else "[dim]available[/dim]"
        table.add_row(entry.name, entry.description[:60], entry.author, entry.version, status)

    console.print(table)


@marketplace_app.command("install")
def marketplace_install(name: str) -> None:
    """Install a connector from the marketplace."""
    from mneia.marketplace.installer import install_connector, is_installed
    from mneia.marketplace.registry import fetch_index

    entries = fetch_index()
    entry = next((e for e in entries if e.name == name), None)

    if not entry:
        console.print(f"[red]Connector '{name}' not found in marketplace.[/red]")
        raise typer.Exit(1)

    if is_installed(entry.package_name):
        console.print(f"[yellow]{entry.package_name} is already installed.[/yellow]")
        return

    console.print(f"[cyan]Installing {entry.package_name}...[/cyan]")
    if install_connector(entry.package_name):
        console.print(f"[green]Installed {entry.display_name}![/green]")
        console.print(f"Enable it with: [cyan]mneia connector enable {name}[/cyan]")
    else:
        console.print("[red]Installation failed. Check logs for details.[/red]")
        raise typer.Exit(1)


@marketplace_app.command("list")
def marketplace_list_cmd() -> None:
    """List all available marketplace connectors."""
    from mneia.marketplace.registry import fetch_index

    entries = fetch_index()
    if not entries:
        console.print("[yellow]No connectors available.[/yellow]")
        return

    table = Table(title="Marketplace Connectors")
    table.add_column("Name", style="cyan")
    table.add_column("Display Name")
    table.add_column("Auth", style="dim")
    table.add_column("Tags", style="dim")
    table.add_column("Status")

    for entry in entries:
        status = "[green]installed[/green]" if entry.installed else "[dim]available[/dim]"
        tags = ", ".join(entry.tags) if entry.tags else ""
        table.add_row(entry.name, entry.display_name, entry.auth_type, tags, status)


# --- Agents TUI ---


@app.command()
def agents() -> None:
    """Interactive TUI dashboard for monitoring agents."""
    from mneia.tui import run_dashboard

    run_dashboard()


@app.command("agent-stats")
def agent_stats(
    agent: str | None = typer.Option(
        None, "--agent", "-a", help="Filter by agent name",
    ),
) -> None:
    """Show agent activity statistics for the last 24 hours."""
    from datetime import datetime

    from mneia.core.agent_stats import AgentStatsDB

    output = get_output()
    db = AgentStatsDB()
    stats = db.get_stats_24h()

    if output.is_json:
        filtered = {}
        for name, events in stats.items():
            if agent and agent not in name:
                continue
            filtered[name] = events
        recent = db.get_recent_events(agent_name=agent, limit=10)
        output.json_result({
            "agents": filtered,
            "recent_events": [
                {
                    "agent": ev.agent_name,
                    "event": ev.event_type,
                    "timestamp": ev.timestamp,
                    "details": ev.details,
                }
                for ev in recent
            ],
        })
        db.close()
        return

    if not stats:
        console.print("[dim]No agent activity in the last 24 hours.[/dim]")
        db.close()
        return

    table = Table(title="Agent Stats (Last 24h)")
    table.add_column("Agent", style="cyan")
    table.add_column("Starts", justify="right")
    table.add_column("Cycles", justify="right")
    table.add_column("Docs Processed", justify="right", style="green")
    table.add_column("Errors", justify="right", style="red")
    table.add_column("Restarts", justify="right", style="yellow")

    for agent_name, events in sorted(stats.items()):
        if agent and agent not in agent_name:
            continue
        table.add_row(
            agent_name,
            str(events.get("start", 0)),
            str(events.get("cycle", 0)),
            str(events.get("docs_processed", 0)),
            str(events.get("error", 0)),
            str(events.get("restart", 0)),
        )

    console.print(table)

    recent = db.get_recent_events(agent_name=agent, limit=10)
    if recent:
        console.print("\n[bold]Recent Events:[/bold]")
        for ev in recent:
            ts = datetime.fromtimestamp(ev.timestamp).strftime("%H:%M:%S")
            details = f" — {ev.details}" if ev.details else ""
            console.print(
                f"  [dim]{ts}[/dim] [{ev.agent_name}] "
                f"{ev.event_type}{details}"
            )

    db.close()


# --- Logs & Update ---


@app.command()
def logs(
    level: str = typer.Option("info", "--level", "-l", help="Log level filter"),
    follow: bool = typer.Option(False, "--follow", "-f", help="Follow log output"),
    lines: int = typer.Option(50, "--lines", "-n", help="Number of lines to show"),
) -> None:
    """Tail daemon logs."""
    import time

    from mneia.config import LOGS_DIR

    log_file = LOGS_DIR / "daemon.log"
    if not log_file.exists():
        console.print("[yellow]No log file found. Start the daemon first.[/yellow]")
        return

    level_upper = level.upper()
    level_priority = {"DEBUG": 0, "INFO": 1, "WARNING": 2, "WARN": 2, "ERROR": 3, "CRITICAL": 4}
    min_priority = level_priority.get(level_upper, 1)

    def should_show(line: str) -> bool:
        for lname, lprio in level_priority.items():
            if lname in line.upper() and lprio >= min_priority:
                return True
        if min_priority <= 1:
            return True
        return False

    with open(str(log_file)) as f:
        all_lines = f.readlines()
        tail = all_lines[-lines:]
        for line in tail:
            if should_show(line):
                console.print(line.rstrip())

        if follow:
            console.print("[dim]Following logs... (Ctrl+C to stop)[/dim]")
            try:
                while True:
                    line = f.readline()
                    if line:
                        if should_show(line):
                            console.print(line.rstrip())
                    else:
                        time.sleep(0.5)
            except KeyboardInterrupt:
                console.print("\n[dim]Stopped.[/dim]")



@app.command("install-skill")
def install_skill(
    no_context: bool = typer.Option(False, "--no-context", help="Skip context generation"),
) -> None:
    """Install the mneia Claude Code skill into ~/.agents/skills/mneia/.

    After installation the skill is available in Claude Code and Claude can
    query your knowledge base directly. Also generates a personal context file
    at ~/.mneia/claude-context.md and adds a reference to ~/.claude/CLAUDE.md.
    """
    import shutil

    skill_src = Path(__file__).parent.parent.parent.parent / "skills" / "claude-code" / "mneia"
    # Fallback: try installed package data path
    if not skill_src.exists():
        import importlib.resources as pkg
        try:
            skill_src = Path(str(pkg.files("mneia"))) / ".." / ".." / "skills" / "claude-code" / "mneia"
        except Exception:
            pass

    if not skill_src.exists() or not (skill_src / "SKILL.md").exists():
        console.print("[red]Skill source not found. Try running from the mneia repo.[/red]")
        raise typer.Exit(1)

    skill_dst = Path.home() / ".agents" / "skills" / "mneia"
    skill_dst.parent.mkdir(parents=True, exist_ok=True)

    if skill_dst.exists():
        shutil.rmtree(skill_dst)
    shutil.copytree(skill_src, skill_dst)
    console.print(f"[green]✓ Skill installed → {skill_dst}[/green]")

    # Generate context
    if not no_context:
        console.print("[cyan]Generating Claude Code context...[/cyan]")
        try:
            ctx = typer.get_current_context()
            ctx.invoke(context_generate_claude)
        except Exception as e:
            console.print(f"[yellow]Context generation skipped: {e}[/yellow]")

    # Pointer in CLAUDE.md
    global_claude = Path.home() / ".claude" / "CLAUDE.md"
    context_file = Path.home() / ".mneia" / "claude-context.md"
    pointer = f"@{context_file}"

    if global_claude.exists():
        existing = global_claude.read_text(encoding="utf-8")
        if str(context_file) not in existing:
            with global_claude.open("a", encoding="utf-8") as f:
                f.write(f"\n\n# mneia personal context\n{pointer}\n")
            console.print("[green]✓ Added mneia context pointer to ~/.claude/CLAUDE.md[/green]")
        else:
            console.print("[dim]CLAUDE.md already references mneia context[/dim]")
    else:
        global_claude.parent.mkdir(parents=True, exist_ok=True)
        global_claude.write_text(f"# mneia personal context\n{pointer}\n", encoding="utf-8")
        console.print("[green]✓ Created ~/.claude/CLAUDE.md with mneia context pointer[/green]")

    console.print()
    console.print("[bold green]mneia skill ready in Claude Code![/bold green]")
    console.print('[dim]Try: "ask mneia what I worked on last week"[/dim]')
    console.print('[dim]Refresh context after syncing: mneia context generate-claude[/dim]')


@app.command()
def update() -> None:
    """Check for updates and install if available."""
    import httpx

    console.print("[cyan]Checking for updates...[/cyan]")
    try:
        resp = httpx.get(
            "https://api.github.com/repos/riverphoenix/mneia/releases/latest",
            timeout=5,
        )
        if resp.status_code == 200:
            latest = resp.json()["tag_name"].lstrip("v")
            if latest != __version__:
                console.print(f"[green]New version available: {latest} (current: {__version__})[/green]")
                console.print("Run: [cyan]pipx upgrade mneia[/cyan]")
            else:
                console.print(f"[green]You're on the latest version ({__version__})[/green]")
        else:
            console.print("[yellow]Could not check for updates.[/yellow]")
    except Exception:
        console.print("[yellow]Could not reach GitHub. Check your connection.[/yellow]")


# --- Entry point ---


def main() -> None:
    try:
        app()
    except SystemExit:
        raise
    except KeyboardInterrupt:
        raise SystemExit(0)
    except Exception as e:
        output = get_output()
        output.error(f"Unexpected error: {e}")
        output.debug("Run with --verbose for full traceback")
        raise SystemExit(EXIT_INTERNAL_ERROR)



from __future__ import annotations

import logging
import subprocess
import sys

logger = logging.getLogger(__name__)


def install_connector(package_name: str) -> bool:
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "install", package_name],
            capture_output=True,
            text=True,
            timeout=120,
        )
        if result.returncode == 0:
            logger.info(f"Installed {package_name}")
            return True
        logger.error(f"pip install failed: {result.stderr}")
        return False
    except subprocess.TimeoutExpired:
        logger.error(f"Installation timed out for {package_name}")
        return False
    except Exception as e:
        logger.error(f"Installation failed: {e}")
        return False


def uninstall_connector(package_name: str) -> bool:
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "uninstall", "-y", package_name],
            capture_output=True,
            text=True,
            timeout=60,
        )
        if result.returncode == 0:
            logger.info(f"Uninstalled {package_name}")
            return True
        logger.error(f"pip uninstall failed: {result.stderr}")
        return False
    except Exception as e:
        logger.error(f"Uninstall failed: {e}")
        return False


def is_installed(package_name: str) -> bool:
    try:
        from importlib.metadata import distribution
        distribution(package_name)
        return True
    except Exception:
        return False



from __future__ import annotations

import asyncio
import logging
from datetime import datetime, timezone
from typing import Any

from mneia.config import MneiaConfig
from mneia.core.agent import AgentResult, AgentState, BaseAgent
from mneia.core.connector import RawDocument
from mneia.core.llm import LLMClient
from mneia.memory.store import MemoryStore

logger = logging.getLogger(__name__)


class WebResearchAgent(BaseAgent):
    def __init__(self, config: MneiaConfig) -> None:
        super().__init__(
            name="web-research",
            description="Deep web research on a given topic",
        )
        self.config = config
        self._llm = LLMClient(config.llm)
        self._store = MemoryStore()

    async def run(self, **kwargs: Any) -> AgentResult:
        self._state = AgentState.RUNNING
        topic = kwargs.get("topic", "")
        max_pages = kwargs.get(
            "max_pages",
            self.config.enrichment_max_scrape_pages,
        )

        if not topic:
            self._state = AgentState.STOPPED
            return AgentResult(
                agent_name=self.name,
                errors=["No topic provided"],
            )

        try:
            result = await self._research(topic, max_pages)
            return result
        except Exception as e:
            logger.exception(f"Web research failed for '{topic}'")
            self._state = AgentState.ERROR
            return AgentResult(
                agent_name=self.name,
                errors=[str(e)],
            )
        finally:
            await self._llm.close()
            if self._state != AgentState.ERROR:
                self._state = AgentState.STOPPED

    async def stop(self) -> None:
        self._state = AgentState.STOPPED

    async def _research(
        self, topic: str, max_pages: int,
    ) -> AgentResult:
        urls = await self._search_urls(topic)
        if not urls:
            return AgentResult(
                agent_name=self.name,
                errors=["No URLs found for topic"],
            )

        scraped_pages = await self._scrape_pages(
            urls[:max_pages],
        )
        if not scraped_pages:
            return AgentResult(
                agent_name=self.name,
                errors=["Could not scrape any pages"],
            )

        summary = await self._synthesize(topic, scraped_pages)

        doc = RawDocument(
            source="web-research",
            source_id=f"research-{topic[:50].replace(' ', '-').lower()}",
            content=summary,
            content_type="research",
            title=f"Research: {topic}",
            timestamp=datetime.now(timezone.utc),
            url=urls[0] if urls else None,
            metadata={
                "topic": topic,
                "pages_scraped": len(scraped_pages),
                "source_urls": urls[:max_pages],
            },
        )

        doc_id = await self._store.store_document(doc)
        logger.info(
            f"Stored research on '{topic}' (doc_id={doc_id}, "
            f"{len(scraped_pages)} pages scraped)"
        )

        return AgentResult(
            agent_name=self.name,
            documents_processed=1,
            metadata={
                "topic": topic,
                "pages_scraped": len(scraped_pages),
                "doc_id": doc_id,
            },
        )

    async def _search_urls(self, topic: str) -> list[str]:
        try:
            import httpx

            async with httpx.AsyncClient(timeout=10) as client:
                resp = await client.get(
                    "https://api.duckduckgo.com/",
                    params={
                        "q": topic,
                        "format": "json",
                        "no_html": 1,
                    },
                )
                if resp.status_code != 200:
                    return []
                data = resp.json()
                urls = []
                abstract_url = data.get("AbstractURL", "")
                if abstract_url:
                    urls.append(abstract_url)
                for t in data.get("RelatedTopics", [])[:10]:
                    if isinstance(t, dict):
                        url = t.get("FirstURL", "")
                        if url and url.startswith("http"):
                            urls.append(url)
                return urls
        except Exception as e:
            logger.warning(f"URL search failed for '{topic}': {e}")
            return []

    async def _scrape_pages(
        self, urls: list[str],
    ) -> list[dict[str, str]]:
        from mneia.connectors.web_scraper import scrape_url

        delay = self.config.enrichment_scrape_delay_seconds
        pages = []
        for url in urls:
            try:
                content = await scrape_url(
                    url, max_content_length=10000,
                )
                if content and len(content) > 50:
                    pages.append({"url": url, "content": content})
                if delay > 0:
                    await asyncio.sleep(delay)
            except Exception as e:
                logger.debug(f"Scrape failed for {url}: {e}")
        return pages

    async def _synthesize(
        self, topic: str, pages: list[dict[str, str]],
    ) -> str:
        context_parts = []
        for page in pages:
            context_parts.append(
                f"[Source: {page['url']}]\n{page['content'][:3000]}"
            )
        context = "\n\n---\n\n".join(context_parts)

        prompt = (
            f"Research topic: {topic}\n\n"
            f"Sources:\n{context}\n\n"
            "Based on these sources, provide a structured research "
            "summary with:\n"
            "1. Overview (2-3 sentences)\n"
            "2. Key findings (bullet points)\n"
            "3. Notable details\n"
            "4. Sources used\n\n"
            "Be factual and cite the sources."
        )

        try:
            return await self._llm.generate(prompt)
        except Exception as e:
            logger.warning(f"Synthesis failed: {e}")
            return "\n\n---\n\n".join(
                f"## {page['url']}\n{page['content'][:1000]}"
                for page in pages
            )



from __future__ import annotations

import asyncio
import logging
from html.parser import HTMLParser

logger = logging.getLogger(__name__)

_RATE_LIMIT_DELAY = 1.0
_last_request_time = 0.0


class _TextExtractor(HTMLParser):
    def __init__(self) -> None:
        super().__init__()
        self._parts: list[str] = []
        self._skip = False
        self._skip_tags = {"script", "style", "noscript", "nav", "footer", "header"}

    def handle_starttag(self, tag: str, attrs: list[tuple[str, str | None]]) -> None:
        if tag in self._skip_tags:
            self._skip = True

    def handle_endtag(self, tag: str) -> None:
        if tag in self._skip_tags:
            self._skip = False
        if tag in ("p", "br", "div", "h1", "h2", "h3", "h4", "h5", "h6", "li"):
            self._parts.append("\n")

    def handle_data(self, data: str) -> None:
        if not self._skip:
            text = data.strip()
            if text:
                self._parts.append(text)

    def get_text(self) -> str:
        raw = " ".join(self._parts)
        lines = [line.strip() for line in raw.split("\n")]
        return "\n".join(line for line in lines if line)


def extract_text_from_html(html: str) -> str:
    parser = _TextExtractor()
    parser.feed(html)
    return parser.get_text()


async def _rate_limit() -> None:
    global _last_request_time
    import time
    now = time.monotonic()
    elapsed = now - _last_request_time
    if elapsed < _RATE_LIMIT_DELAY:
        await asyncio.sleep(_RATE_LIMIT_DELAY - elapsed)
    _last_request_time = time.monotonic()


async def scrape_url(url: str, max_content_length: int = 50000) -> str | None:
    content = await _scrape_crawl4ai(url)
    if content:
        return content[:max_content_length]

    content = await _scrape_httpx(url)
    if content:
        return content[:max_content_length]

    return None


async def _scrape_crawl4ai(url: str) -> str | None:
    try:
        from crawl4ai import AsyncWebCrawler
    except ImportError:
        return None

    try:
        await _rate_limit()
        async with AsyncWebCrawler() as crawler:
            result = await crawler.arun(url=url)
            if result and result.markdown:
                return result.markdown
    except Exception as e:
        logger.debug(f"crawl4ai failed for {url}: {e}")
    return None


async def _scrape_httpx(url: str) -> str | None:
    try:
        import httpx
    except ImportError:
        return None

    try:
        await _rate_limit()
        async with httpx.AsyncClient(
            timeout=15,
            follow_redirects=True,
        ) as client:
            resp = await client.get(
                url,
                headers={
                    "User-Agent": (
                        "Mozilla/5.0 (compatible; mneia/0.1; "
                        "+https://github.com/riverphoenix/mneia)"
                    ),
                },
            )
            if resp.status_code != 200:
                return None
            content_type = resp.headers.get("content-type", "")
            if "text/html" not in content_type:
                return None
            return extract_text_from_html(resp.text)
    except Exception as e:
        logger.debug(f"httpx scrape failed for {url}: {e}")
    return None



from __future__ import annotations

import json
import sqlite3
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

import networkx as nx

from mneia.config import DATA_DIR

GRAPH_DB_PATH = DATA_DIR / "graph.db"

GRAPH_SCHEMA = """
CREATE TABLE IF NOT EXISTS graph_nodes (
    id TEXT PRIMARY KEY,
    entity_type TEXT NOT NULL,
    name TEXT NOT NULL,
    properties TEXT DEFAULT '{}'
);

CREATE TABLE IF NOT EXISTS graph_edges (
    source_id TEXT NOT NULL,
    target_id TEXT NOT NULL,
    relation TEXT NOT NULL,
    weight REAL DEFAULT 1.0,
    evidence TEXT DEFAULT '',
    PRIMARY KEY (source_id, target_id, relation),
    FOREIGN KEY (source_id) REFERENCES graph_nodes(id),
    FOREIGN KEY (target_id) REFERENCES graph_nodes(id)
);
"""


@dataclass
class GraphNode:
    id: str
    entity_type: str
    name: str
    properties: dict[str, Any] = field(default_factory=dict)
    first_seen: str | None = None
    last_seen: str | None = None
    mention_count: int = 1


@dataclass
class GraphEdge:
    source_id: str
    target_id: str
    relation: str
    weight: float = 1.0
    evidence: str = ""
    first_seen: str | None = None
    last_seen: str | None = None


class KnowledgeGraph:
    def __init__(self, db_path: Path | None = None) -> None:
        self._db_path = db_path or GRAPH_DB_PATH
        self._db_path.parent.mkdir(parents=True, exist_ok=True)
        self._graph = nx.DiGraph()
        self._init_db()
        self._load_from_db()

    def _get_conn(self) -> sqlite3.Connection:
        conn = sqlite3.connect(str(self._db_path))
        conn.row_factory = sqlite3.Row
        return conn

    def _init_db(self) -> None:
        conn = self._get_conn()
        try:
            conn.executescript(GRAPH_SCHEMA)
            conn.commit()
            self._migrate_schema(conn)
        finally:
            conn.close()

    def _migrate_schema(self, conn: sqlite3.Connection) -> None:
        cols = {row[1] for row in conn.execute("PRAGMA table_info(graph_nodes)")}
        if "first_seen" not in cols:
            conn.execute("ALTER TABLE graph_nodes ADD COLUMN first_seen TEXT")
            conn.execute("ALTER TABLE graph_nodes ADD COLUMN last_seen TEXT")
            conn.execute("ALTER TABLE graph_nodes ADD COLUMN mention_count INTEGER DEFAULT 1")
            conn.commit()
        edge_cols = {row[1] for row in conn.execute("PRAGMA table_info(graph_edges)")}
        if "first_seen" not in edge_cols:
            conn.execute("ALTER TABLE graph_edges ADD COLUMN first_seen TEXT")
            conn.execute("ALTER TABLE graph_edges ADD COLUMN last_seen TEXT")
            conn.commit()

    def _load_from_db(self) -> None:
        conn = self._get_conn()
        try:
            for row in conn.execute("SELECT * FROM graph_nodes"):
                self._graph.add_node(
                    row["id"],
                    entity_type=row["entity_type"],
                    name=row["name"],
                    properties=json.loads(row["properties"]),
                )
            for row in conn.execute("SELECT * FROM graph_edges"):
                self._graph.add_edge(
                    row["source_id"],
                    row["target_id"],
                    relation=row["relation"],
                    weight=row["weight"],
                    evidence=row["evidence"],
                )
        finally:
            conn.close()

    def add_entity(self, node: GraphNode) -> None:
        from datetime import datetime, timezone

        now = datetime.now(timezone.utc).isoformat()
        self._graph.add_node(
            node.id,
            entity_type=node.entity_type,
            name=node.name,
            properties=node.properties,
            first_seen=node.first_seen or now,
            last_seen=now,
            mention_count=node.mention_count,
        )
        conn = self._get_conn()
        try:
            conn.execute(
                """
                INSERT INTO graph_nodes (id, entity_type, name, properties, first_seen, last_seen, mention_count)
                VALUES (?, ?, ?, ?, ?, ?, 1)
                ON CONFLICT(id) DO UPDATE SET
                    name = excluded.name,
                    properties = excluded.properties,
                    last_seen = excluded.last_seen,
                    mention_count = COALESCE(mention_count, 0) + 1
                """,
                (node.id, node.entity_type, node.name, json.dumps(node.properties), now, now),
            )
            conn.commit()
        finally:
            conn.close()

    def add_relationship(self, edge: GraphEdge) -> None:
        from datetime import datetime, timezone

        now = datetime.now(timezone.utc).isoformat()
        self._graph.add_edge(
            edge.source_id,
            edge.target_id,
            relation=edge.relation,
            weight=edge.weight,
            evidence=edge.evidence,
            first_seen=edge.first_seen or now,
            last_seen=now,
        )
        conn = self._get_conn()
        try:
            conn.execute(
                """
                INSERT INTO graph_edges (source_id, target_id, relation, weight, evidence, first_seen, last_seen)
                VALUES (?, ?, ?, ?, ?, ?, ?)
                ON CONFLICT(source_id, target_id, relation) DO UPDATE SET
                    weight = weight + excluded.weight,
                    evidence = excluded.evidence,
                    last_seen = excluded.last_seen
                """,
                (edge.source_id, edge.target_id, edge.relation, edge.weight, edge.evidence, now, now),
            )
            conn.commit()
        finally:
            conn.close()

    def remove_node(self, node_id: str) -> None:
        if node_id in self._graph:
            self._graph.remove_node(node_id)
        conn = self._get_conn()
        try:
            conn.execute("DELETE FROM graph_edges WHERE source_id = ? OR target_id = ?", (node_id, node_id))
            conn.execute("DELETE FROM graph_nodes WHERE id = ?", (node_id,))
            conn.commit()
        finally:
            conn.close()

    def update_node_properties(self, node_id: str, properties: dict[str, Any]) -> None:
        if node_id not in self._graph:
            return
        self._graph.nodes[node_id]["properties"] = properties
        conn = self._get_conn()
        try:
            conn.execute(
                "UPDATE graph_nodes SET properties = ? WHERE id = ?",
                (json.dumps(properties), node_id),
            )
            conn.commit()
        finally:
            conn.close()

    def get_entities_by_type(self, entity_type: str) -> list[dict[str, Any]]:
        results = []
        for nid, data in self._graph.nodes(data=True):
            if data.get("entity_type") == entity_type:
                results.append({"id": nid, "name": data.get("name", ""), **data.get("properties", {})})
        return results

    def get_neighbors(self, node_id: str, depth: int = 1) -> dict[str, Any]:
        if node_id not in self._graph:
            return {"nodes": [], "edges": []}

        nodes_at_depth = {node_id}
        all_nodes = {node_id}
        for _ in range(depth):
            next_level: set[str] = set()
            for n in nodes_at_depth:
                next_level.update(self._graph.successors(n))
                next_level.update(self._graph.predecessors(n))
            nodes_at_depth = next_level - all_nodes
            all_nodes.update(nodes_at_depth)

        subgraph = self._graph.subgraph(all_nodes)
        nodes = []
        for n in subgraph.nodes(data=True):
            nodes.append({
                "id": n[0],
                "name": n[1].get("name", ""),
                "type": n[1].get("entity_type", ""),
            })

        edges = []
        for e in subgraph.edges(data=True):
            edges.append({
                "source": e[0],
                "target": e[1],
                "relation": e[2].get("relation", ""),
                "weight": e[2].get("weight", 1.0),
            })

        return {"nodes": nodes, "edges": edges}

    def get_stats(self) -> dict[str, Any]:
        type_counts: dict[str, int] = {}
        for _, data in self._graph.nodes(data=True):
            t = data.get("entity_type", "unknown")
            type_counts[t] = type_counts.get(t, 0) + 1

        return {
            "total_nodes": self._graph.number_of_nodes(),
            "total_edges": self._graph.number_of_edges(),
            "by_type": type_counts,
        }

    def get_recent_entities(self, since: str) -> list[dict[str, Any]]:
        conn = self._get_conn()
        try:
            rows = conn.execute(
                "SELECT * FROM graph_nodes WHERE last_seen >= ? ORDER BY last_seen DESC",
                (since,),
            ).fetchall()
            return [dict(r) for r in rows]
        finally:
            conn.close()

    def get_trending_entities(self, limit: int = 10) -> list[dict[str, Any]]:
        conn = self._get_conn()
        try:
            rows = conn.execute(
                "SELECT * FROM graph_nodes ORDER BY mention_count DESC LIMIT ?",
                (limit,),
            ).fetchall()
            return [dict(r) for r in rows]
        finally:
            conn.close()

    def export_json(self) -> dict[str, Any]:
        nodes = []
        for n, data in self._graph.nodes(data=True):
            nodes.append({"id": n, **data})
        edges = []
        for s, t, data in self._graph.edges(data=True):
            edges.append({"source": s, "target": t, **data})
        return {"nodes": nodes, "edges": edges}



from __future__ import annotations

import logging
import platform
import shutil
import sqlite3
import tempfile
from datetime import datetime, timezone
from pathlib import Path
from typing import Any, AsyncIterator

from mneia.core.connector import (
    BaseConnector,
    ConnectorManifest,
    ConnectorMode,
    RawDocument,
)

logger = logging.getLogger(__name__)

CHROME_EPOCH = datetime(1601, 1, 1, tzinfo=timezone.utc)


def _default_chrome_history_path() -> Path | None:
    system = platform.system()
    home = Path.home()
    if system == "Darwin":
        return home / "Library/Application Support/Google/Chrome/Default/History"
    if system == "Linux":
        return home / ".config/google-chrome/Default/History"
    if system == "Windows":
        return home / "AppData/Local/Google/Chrome/User Data/Default/History"
    return None


def _find_chrome_history() -> list[Path]:
    """Search Chrome/Brave/Edge profile directories and return existing History files, largest first."""
    import glob as _glob

    system = platform.system()
    home = Path.home()
    found: list[Path] = []

    if system == "Darwin":
        base_dirs = [
            home / "Library/Application Support/Google/Chrome",
            home / "Library/Application Support/Google/Chrome Beta",
            home / "Library/Application Support/Google/Chrome Canary",
            home / "Library/Application Support/BraveSoftware/Brave-Browser",
            home / "Library/Application Support/Microsoft Edge",
            home / "Library/Application Support/Chromium",
        ]
        user_data_dirs = [
            home / "Library/Application Support/Google/Chrome/User Data",
        ]
    elif system == "Linux":
        base_dirs = [
            home / ".config/google-chrome",
            home / ".config/google-chrome-beta",
            home / ".config/chromium",
            home / ".config/BraveSoftware/Brave-Browser",
        ]
        user_data_dirs = []
    elif system == "Windows":
        local = home / "AppData/Local"
        base_dirs = [
            local / "Google/Chrome/User Data",
            local / "BraveSoftware/Brave-Browser/User Data",
            local / "Microsoft/Edge/User Data",
            local / "Chromium/User Data",
        ]
        user_data_dirs = base_dirs[:]
    else:
        return []

    # Collect History files from Default + Profile N subdirs
    search_roots = base_dirs + user_data_dirs
    for base in search_roots:
        if not base.exists():
            continue
        for pattern in ["Default/History", "Profile */History", "Profile*/History"]:
            for p in sorted(base.glob(pattern)):
                if p not in found:
                    found.append(p)

    # Sort by file size descending so the most-used profile comes first
    found = [p for p in found if p.exists()]
    found.sort(key=lambda p: p.stat().st_size, reverse=True)
    return found


def _chrome_time_to_datetime(chrome_ts: int) -> datetime:
    if chrome_ts == 0:
        return datetime.now(timezone.utc)
    seconds = chrome_ts / 1_000_000
    return CHROME_EPOCH + __import__("datetime").timedelta(seconds=seconds)


class ChromeHistoryConnector(BaseConnector):
    manifest = ConnectorManifest(
        name="chrome-history",
        display_name="Chrome History",
        version="0.1.0",
        description="Read browsing history from Google Chrome (read-only, local)",
        author="mneia-team",
        mode=ConnectorMode.POLL,
        auth_type="local",
        scopes=["read"],
        poll_interval_seconds=600,
        required_config=[],
        optional_config=[
            "history_path", "max_results",
            "scrape_content", "scrape_max_pages",
            "scrape_domains_exclude",
        ],
    )

    def __init__(self) -> None:
        self._history_path: Path | None = None
        self._max_results: int = 200
        self._scrape_content: bool = False
        self._scrape_max_pages: int = 20
        self._scrape_domains_exclude: set[str] = set()

    async def authenticate(self, config: dict[str, Any]) -> bool:
        path_str = config.get("history_path", "")
        if path_str:
            self._history_path = Path(path_str)
        else:
            default = _default_chrome_history_path()
            if default and default.exists():
                self._history_path = default
            else:
                found = _find_chrome_history()
                self._history_path = found[0] if found else default

        if not self._history_path or not self._history_path.exists():
            logger.error(f"Chrome history not found at: {self._history_path}")
            return False

        max_r = config.get("max_results", "")
        if max_r:
            self._max_results = int(max_r)

        scrape = config.get("scrape_content", "")
        if isinstance(scrape, bool):
            self._scrape_content = scrape
        elif isinstance(scrape, str) and scrape:
            self._scrape_content = scrape.lower() in (
                "true", "1", "yes",
            )

        max_pages = config.get("scrape_max_pages", "")
        if max_pages:
            self._scrape_max_pages = int(max_pages)

        exclude = config.get("scrape_domains_exclude", "")
        if exclude:
            self._scrape_domains_exclude = {
                d.strip()
                for d in exclude.split(",")
                if d.strip()
            }

        return True

    async def fetch_since(self, since: datetime | None) -> AsyncIterator[RawDocument]:
        if not self._history_path:
            return

        tmp_dir = tempfile.mkdtemp()
        tmp_path = Path(tmp_dir) / "History"
        try:
            shutil.copy2(self._history_path, tmp_path)
        except (OSError, PermissionError) as e:
            logger.error(f"Cannot copy Chrome history: {e}")
            shutil.rmtree(tmp_dir, ignore_errors=True)
            return

        try:
            conn = sqlite3.connect(f"file:{tmp_path}?mode=ro", uri=True)
            conn.row_factory = sqlite3.Row

            query = """
                SELECT u.id, u.url, u.title, u.visit_count, u.last_visit_time,
                       u.typed_count
                FROM urls u
            """
            params: list[Any] = []

            if since:
                chrome_ts = int((since - CHROME_EPOCH).total_seconds() * 1_000_000)
                query += " WHERE u.last_visit_time > ?"
                params.append(chrome_ts)

            query += " ORDER BY u.last_visit_time DESC LIMIT ?"
            params.append(self._max_results)

            cursor = conn.execute(query, params)
            rows = cursor.fetchall()
            conn.close()

            scraped_count = 0
            for row in rows:
                doc = self._row_to_document(row)
                if doc:
                    yield doc

                    if (
                        self._scrape_content
                        and doc.url
                        and scraped_count < self._scrape_max_pages
                        and not self._is_excluded_domain(doc.url)
                        and not doc.metadata.get("content_scraped")
                    ):
                        scraped = await self._scrape_page(doc)
                        if scraped:
                            scraped_count += 1
                            yield scraped
        except Exception as e:
            logger.error(f"Chrome history read failed: {e}")
        finally:
            shutil.rmtree(tmp_dir, ignore_errors=True)

    async def health_check(self) -> bool:
        if not self._history_path:
            return False
        return self._history_path.exists()

    def interactive_setup(self) -> dict[str, Any]:
        import typer

        found = _find_chrome_history()
        if not found:
            typer.echo("\n  No Chrome-based browser history found at standard locations.")
            typer.echo("  Please enter the full path to your browser's History file.")
            typer.echo("  (e.g. ~/Library/Application Support/Google/Chrome/Default/History)")
            path = typer.prompt("  History file path")
            return {"history_path": path}

        if len(found) == 1:
            typer.echo(f"\n  Found browser history: {found[0]}")
            return {"history_path": str(found[0])}

        typer.echo("\n  Found multiple browser history files:")
        for i, p in enumerate(found):
            typer.echo(f"  [{i}] {p}")
        choice = typer.prompt("  Select browser (number or press Enter for first)", default="0")
        try:
            idx = int(choice)
            if 0 <= idx < len(found):
                return {"history_path": str(found[idx])}
        except ValueError:
            if choice.strip():
                return {"history_path": choice.strip()}
        return {"history_path": str(found[0])}

    def _is_excluded_domain(self, url: str) -> bool:
        from urllib.parse import urlparse

        try:
            domain = urlparse(url).netloc.lower()
        except Exception:
            return True
        if not domain:
            return True
        for excluded in self._scrape_domains_exclude:
            if excluded in domain:
                return True
        internal_prefixes = (
            "chrome://", "chrome-extension://",
            "about:", "file://",
        )
        if any(url.startswith(p) for p in internal_prefixes):
            return True
        return False

    async def _scrape_page(
        self, doc: RawDocument,
    ) -> RawDocument | None:
        if not doc.url:
            return None
        try:
            from mneia.connectors.web_scraper import scrape_url

            content = await scrape_url(doc.url)
            if not content or len(content) < 50:
                return None

            return RawDocument(
                source="chrome-history",
                source_id=f"scraped-{doc.source_id}",
                content=content,
                content_type="webpage",
                title=f"{doc.title} (content)",
                timestamp=doc.timestamp,
                url=doc.url,
                metadata={
                    **doc.metadata,
                    "content_scraped": True,
                    "original_source_id": doc.source_id,
                },
            )
        except Exception as e:
            logger.debug(f"Scrape failed for {doc.url}: {e}")
            return None

    @staticmethod
    def _row_to_document(row: sqlite3.Row) -> RawDocument | None:
        url = row["url"]
        title = row["title"] or url
        if not url:
            return None

        visit_count = row["visit_count"] or 0
        typed_count = row["typed_count"] or 0
        last_visit = row["last_visit_time"] or 0

        timestamp = _chrome_time_to_datetime(last_visit)

        content_parts = [f"# {title}", f"URL: {url}"]
        content_parts.append(f"Visits: {visit_count}")
        if typed_count:
            content_parts.append(f"Typed: {typed_count}")

        return RawDocument(
            source="chrome-history",
            source_id=str(row["id"]),
            content="\n".join(content_parts),
            content_type="bookmark",
            title=title,
            timestamp=timestamp,
            url=url,
            metadata={
                "visit_count": visit_count,
                "typed_count": typed_count,
            },
        )



from __future__ import annotations

import asyncio
import logging
import shutil
import tempfile
import wave
from collections.abc import AsyncIterator
from datetime import datetime, timezone
from pathlib import Path
from typing import Any

from mneia.connectors.transcription_engine import detect_backend, transcribe
from mneia.core.connector import (
    BaseConnector,
    ConnectorManifest,
    ConnectorMode,
    RawDocument,
)

logger = logging.getLogger(__name__)

CHUNK_DURATION_SECONDS = 30
SAMPLE_RATE = 16000
CHANNELS = 1
SAMPLE_WIDTH = 2


class LiveAudioConnector(BaseConnector):
    manifest = ConnectorManifest(
        name="live-audio",
        display_name="Live Audio Capture",
        version="0.2.0",
        description=(
            "Capture system audio during meetings and transcribe "
            "in real-time using ScreenCaptureKit (macOS 13+) "
            "or virtual audio device"
        ),
        author="mneia-team",
        mode=ConnectorMode.WATCH,
        auth_type="local",
        scopes=["audio:capture"],
        poll_interval_seconds=0,
        required_config=[],
        optional_config=[
            "audio_device",
            "whisper_model",
            "language",
            "chunk_seconds",
            "capture_method",
        ],
    )

    def __init__(self) -> None:
        self._audio_device: str | None = None
        self._whisper_model: str = "base"
        self._language: str = "en"
        self._chunk_seconds: int = CHUNK_DURATION_SECONDS
        self._backend: str = "auto"
        self._capture_method: str = "auto"
        self._recording: bool = False
        self._meeting_start: datetime | None = None
        self._chunk_index: int = 0

    async def authenticate(self, config: dict[str, Any]) -> bool:
        self.last_error = ""
        self._audio_device = config.get("audio_device")
        self._whisper_model = config.get("whisper_model", "base")
        self._language = config.get("language", "en")
        self._capture_method = config.get("capture_method", "auto")

        chunk_s = config.get("chunk_seconds", "")
        if chunk_s:
            self._chunk_seconds = int(chunk_s)

        self._backend = detect_backend()
        if self._backend == "none":
            self.last_error = (
                "No whisper backend found. "
                "Install faster-whisper or whisper-cpp"
            )
            logger.error(self.last_error)
            return False

        if self._capture_method == "auto":
            self._capture_method = _detect_capture_method()

        if self._capture_method == "screencapturekit":
            from mneia.connectors.screencapturekit_audio import (
                compile_capture_binary,
            )
            binary = compile_capture_binary()
            if not binary:
                self.last_error = (
                    "ScreenCaptureKit setup failed. "
                    "Ensure Xcode CLI tools are installed: "
                    "xcode-select --install"
                )
                logger.error(self.last_error)
                return False
            logger.info("Audio capture: ScreenCaptureKit (no virtual device needed)")
        elif self._capture_method == "sounddevice":
            try:
                import sounddevice  # noqa: F401
            except ImportError:
                self.last_error = (
                    "sounddevice not installed. "
                    "Run: pip install 'mneia[audio]'"
                )
                logger.error(self.last_error)
                return False
            logger.info(
                f"Audio capture: sounddevice "
                f"(device={self._audio_device or 'default'})"
            )
        else:
            self.last_error = (
                f"Unknown capture method: {self._capture_method}"
            )
            return False

        return True

    async def fetch_since(
        self, since: datetime | None,
    ) -> AsyncIterator[RawDocument]:
        return
        yield  # noqa: RET504

    async def start_recording(self) -> AsyncIterator[RawDocument]:
        self._recording = True
        self._meeting_start = datetime.now(timezone.utc)
        self._chunk_index = 0

        logger.info(
            f"Live audio capture started "
            f"(method={self._capture_method}, "
            f"chunk={self._chunk_seconds}s)"
        )

        try:
            async for doc in self._capture_loop():
                yield doc
        finally:
            self._recording = False
            logger.info("Live audio capture stopped")

    async def stop_recording(self) -> None:
        self._recording = False

    async def health_check(self) -> bool:
        backend = detect_backend()
        if backend == "none":
            return False
        method = _detect_capture_method()
        if method == "screencapturekit":
            return True
        return _check_sounddevice()

    def interactive_setup(self) -> dict[str, Any]:
        import typer

        method = _detect_capture_method()

        if method == "screencapturekit":
            typer.echo(
                "\n  Live Audio Capture — ScreenCaptureKit"
            )
            typer.echo(
                "  Uses macOS ScreenCaptureKit to capture "
                "system audio directly."
            )
            typer.echo(
                "  No virtual audio device needed.\n"
            )
            typer.echo(
                "  On first use, macOS will ask you to grant "
                "Screen Recording permission."
            )
            typer.echo(
                "  Go to System Settings → Privacy & Security "
                "→ Screen Recording → enable mneia.\n"
            )
        else:
            typer.echo(
                "\n  Live Audio Capture — Virtual Audio Device"
            )
            typer.echo(
                "  macOS: Install BlackHole for virtual "
                "audio device."
            )
            typer.echo(
                "  Linux: Use PulseAudio monitor source.\n"
            )

        model = typer.prompt("  Whisper model", default="base")
        language = typer.prompt("  Language", default="en")
        chunk = typer.prompt(
            "  Chunk duration (seconds)", default="30",
        )

        settings: dict[str, Any] = {
            "whisper_model": model,
            "language": language,
            "chunk_seconds": chunk,
            "capture_method": method,
        }

        if method == "sounddevice":
            device = typer.prompt(
                "  Audio device name (or empty for default)",
                default="",
            )
            if device:
                settings["audio_device"] = device

        return settings

    async def _capture_loop(self) -> AsyncIterator[RawDocument]:
        while self._recording:
            tmp_dir = tempfile.mkdtemp(prefix="mneia-audio-")
            wav_path = Path(tmp_dir) / f"chunk_{self._chunk_index}.wav"

            try:
                if self._capture_method == "screencapturekit":
                    recorded = await self._record_sck(wav_path)
                else:
                    recorded = await self._record_sounddevice(wav_path)

                if not recorded:
                    await asyncio.sleep(1)
                    continue

                text = await asyncio.to_thread(
                    transcribe,
                    wav_path,
                    backend=self._backend,
                    model=self._whisper_model,
                    language=self._language,
                )

                if text and len(text.strip()) > 10:
                    self._chunk_index += 1
                    yield self._create_document(text)
            except Exception as e:
                logger.error(f"Capture chunk failed: {e}")
                await asyncio.sleep(1)
            finally:
                shutil.rmtree(tmp_dir, ignore_errors=True)

    async def _record_sck(self, output_path: Path) -> bool:
        from mneia.connectors.screencapturekit_audio import (
            record_system_audio,
        )
        return await record_system_audio(
            output_path,
            duration_seconds=self._chunk_seconds,
            sample_rate=SAMPLE_RATE,
        )

    async def _record_sounddevice(self, output_path: Path) -> bool:
        try:
            import sounddevice as sd
        except ImportError:
            logger.error("sounddevice not installed")
            return False

        try:
            device_index = None
            if self._audio_device:
                devices = sd.query_devices()
                for i, dev in enumerate(devices):
                    if self._audio_device.lower() in dev["name"].lower():
                        device_index = i
                        break

            frames = await asyncio.to_thread(
                sd.rec,
                int(self._chunk_seconds * SAMPLE_RATE),
                samplerate=SAMPLE_RATE,
                channels=CHANNELS,
                dtype="int16",
                device=device_index,
            )
            await asyncio.to_thread(sd.wait)

            with wave.open(str(output_path), "wb") as wf:
                wf.setnchannels(CHANNELS)
                wf.setsampwidth(SAMPLE_WIDTH)
                wf.setframerate(SAMPLE_RATE)
                wf.writeframes(frames.tobytes())

            return True
        except Exception as e:
            logger.error(f"Recording failed: {e}")
            return False

    def _create_document(self, text: str) -> RawDocument:
        now = datetime.now(timezone.utc)
        meeting_id = (
            self._meeting_start.strftime("%Y%m%d-%H%M%S")
            if self._meeting_start
            else "unknown"
        )

        return RawDocument(
            source="live-audio",
            source_id=f"live-{meeting_id}-{self._chunk_index}",
            content=text,
            content_type="live_transcript",
            title=(
                f"Live transcript chunk {self._chunk_index} "
                f"({meeting_id})"
            ),
            timestamp=now,
            metadata={
                "meeting_id": meeting_id,
                "chunk_index": self._chunk_index,
                "meeting_start": (
                    self._meeting_start.isoformat()
                    if self._meeting_start
                    else None
                ),
                "chunk_duration_seconds": self._chunk_seconds,
                "whisper_model": self._whisper_model,
                "capture_method": self._capture_method,
            },
        )


def _detect_capture_method() -> str:
    from mneia.connectors.screencapturekit_audio import is_available

    if is_available():
        return "screencapturekit"
    if _check_sounddevice():
        return "sounddevice"
    return "screencapturekit"


def _check_sounddevice() -> bool:
    try:
        import sounddevice  # noqa: F401
        return True
    except ImportError:
        return False



"""Interactive CLI knowledge graph visualizer.

Uses Rich (already a dependency) — no additional packages required.
Navigation is menu-driven: users can browse entities by type, explore
relationship trees, view the graph overview, and search entities.

Optional: `pip install textual` for a richer TUI panel (auto-detected).
"""

from __future__ import annotations

from typing import Any

from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text
from rich.tree import Tree

console = Console()

_TYPE_COLORS: dict[str, str] = {
    "person": "bright_cyan",
    "project": "bright_yellow",
    "topic": "bright_green",
    "decision": "bright_magenta",
    "belief": "bright_white",
    "meeting": "bright_blue",
    "tool": "green",
    "organization": "bright_red",
}

_ENTITY_TYPES = (
    "person", "project", "topic", "decision",
    "belief", "meeting", "tool", "organization",
)


def _color(entity_type: str) -> str:
    return _TYPE_COLORS.get(entity_type, "white")


def _ask(prompt: str) -> str:
    try:
        return input(f"\n  {prompt}\n  → ").strip().lower()
    except (EOFError, KeyboardInterrupt):
        return "q"


# ── views ─────────────────────────────────────────────────────────────────────

def _show_overview(graph: Any) -> None:
    """Print graph-wide statistics and top entities."""
    stats = graph.get_stats()
    total_nodes = stats["total_nodes"]
    total_edges = stats["total_edges"]
    by_type: dict[str, int] = stats.get("by_type", {})

    console.print()
    console.print(Panel(
        f"[bold]Knowledge Graph Overview[/bold]\n\n"
        f"  Entities  : [cyan]{total_nodes}[/cyan]\n"
        f"  Relations : [cyan]{total_edges}[/cyan]",
        border_style="cyan",
        padding=(1, 2),
    ))

    # Type breakdown table
    if by_type:
        tbl = Table(show_header=True, header_style="bold dim", box=None, padding=(0, 2))
        tbl.add_column("Type")
        tbl.add_column("Count", justify="right")
        tbl.add_column("Bar")
        max_count = max(by_type.values()) or 1
        for etype in _ENTITY_TYPES:
            cnt = by_type.get(etype, 0)
            if cnt == 0:
                continue
            bar_len = int(cnt / max_count * 20)
            bar = "█" * bar_len
            col = _color(etype)
            tbl.add_row(f"[{col}]{etype}[/{col}]", str(cnt), f"[{col}]{bar}[/{col}]")
        console.print(tbl)

    # Top entities by mention_count
    trending = graph.get_trending_entities(limit=10)
    if trending:
        console.print()
        console.print("  [bold dim]Most mentioned:[/bold dim]")
        for row in trending:
            try:
                props = row.get("properties", "{}")
                if isinstance(props, str):
                    props = __import__("json").loads(props)
            except Exception:
                props = {}
            etype = row.get("entity_type", "")
            name = row.get("name", row.get("id", "?"))
            mentions = row.get("mention_count", 1)
            col = _color(etype)
            console.print(
                f"    [{col}]{name}[/{col}] [dim]({etype}, {mentions}✕)[/dim]"
            )
    console.print()


def _show_entity_list(graph: Any, entity_type: str | None) -> list[tuple[str, dict[str, Any]]]:
    """Show a paginated table of entities.  Returns list of (node_id, data)."""
    if entity_type:
        nodes = [
            (nid, data)
            for nid, data in graph._graph.nodes(data=True)
            if data.get("entity_type") == entity_type
        ]
    else:
        nodes = list(graph._graph.nodes(data=True))

    # Sort by mention_count desc then name
    nodes.sort(key=lambda x: (-x[1].get("mention_count", 1), x[1].get("name", "")))

    if not nodes:
        console.print("  [dim]No entities found.[/dim]")
        return []

    label = entity_type or "all types"
    tbl = Table(
        show_header=True,
        header_style="bold dim",
        box=None,
        padding=(0, 2),
        title=f"Entities — {label} ({len(nodes)})",
    )
    tbl.add_column("#", justify="right", style="dim")
    tbl.add_column("Name")
    tbl.add_column("Type")
    tbl.add_column("Links", justify="right")
    tbl.add_column("Mentions", justify="right")
    tbl.add_column("Description", no_wrap=False, max_width=50)

    for i, (nid, data) in enumerate(nodes[:40], 1):
        name = data.get("name", nid)
        etype = data.get("entity_type", "")
        degree = graph._graph.degree(nid)
        mentions = data.get("mention_count", 1)
        props = data.get("properties", {})
        desc = props.get("description", "") if isinstance(props, dict) else ""
        col = _color(etype)
        tbl.add_row(
            str(i),
            f"[{col}]{name}[/{col}]",
            f"[dim]{etype}[/dim]",
            str(degree),
            str(mentions),
            f"[dim]{desc[:60]}[/dim]" if desc else "",
        )

    console.print()
    console.print(tbl)
    if len(nodes) > 40:
        console.print(f"  [dim](showing 40 of {len(nodes)})[/dim]")
    console.print()
    return nodes[:40]


def _build_relation_tree(graph: Any, node_id: str, depth: int = 2) -> Tree:
    """Build a Rich Tree showing relationships up to *depth* hops."""
    data = graph._graph.nodes.get(node_id, {})
    name = data.get("name", node_id)
    etype = data.get("entity_type", "")
    col = _color(etype)

    root = Tree(f"[bold {col}]{name}[/bold {col}] [dim]({etype})[/dim]")
    props = data.get("properties", {})
    if isinstance(props, dict):
        desc = props.get("description", "")
        if desc:
            root.add(f"[dim italic]{desc[:120]}[/dim italic]")

    _add_tree_edges(graph, root, node_id, depth=depth, visited={node_id})
    return root


def _add_tree_edges(
    graph: Any,
    tree: Tree,
    node_id: str,
    depth: int,
    visited: set[str],
) -> None:
    if depth == 0:
        return

    out_edges = [
        (tgt, graph._graph.edges[node_id, tgt])
        for tgt in graph._graph.successors(node_id)
        if tgt not in visited
    ]
    in_edges = [
        (src, graph._graph.edges[src, node_id])
        for src in graph._graph.predecessors(node_id)
        if src not in visited
    ]
    all_edges = out_edges + in_edges

    # Sort by edge weight
    all_edges.sort(key=lambda x: x[1].get("weight", 1.0), reverse=True)

    for other_id, edata in all_edges[:8]:
        other_data = graph._graph.nodes.get(other_id, {})
        other_name = other_data.get("name", other_id)
        other_type = other_data.get("entity_type", "")
        rel = edata.get("relation", "related_to")
        weight = edata.get("weight", 1.0)
        col = _color(other_type)
        w_str = f" [dim]×{weight:.0f}[/dim]" if weight > 1.5 else ""
        branch = tree.add(
            f"[cyan]{rel}[/cyan] → [{col}]{other_name}[/{col}] [dim]({other_type})[/dim]{w_str}"
        )
        if depth > 1:
            _add_tree_edges(graph, branch, other_id, depth - 1, visited | {node_id, other_id})


def _explore_entity(graph: Any, node_id: str) -> str | None:
    """Show full entity detail + relationship tree.  Returns next node_id or None."""
    data = graph._graph.nodes.get(node_id, {})
    if not data:
        console.print(f"  [yellow]Entity not found: {node_id}[/yellow]")
        return None

    name = data.get("name", node_id)
    etype = data.get("entity_type", "")
    props = data.get("properties", {})
    if not isinstance(props, dict):
        props = {}

    degree = graph._graph.degree(node_id)
    mentions = data.get("mention_count", 1)
    first_seen = data.get("first_seen", "")[:10]
    last_seen = data.get("last_seen", "")[:10]

    col = _color(etype)

    # Entity card
    meta_parts = [f"type: {etype}", f"connections: {degree}", f"mentions: {mentions}"]
    if first_seen:
        meta_parts.append(f"first seen: {first_seen}")
    if last_seen:
        meta_parts.append(f"last seen: {last_seen}")

    props_lines = []
    for k, v in props.items():
        if k not in ("description", "source", "doc_id") and v:
            props_lines.append(f"  [dim]{k}:[/dim] {str(v)[:100]}")

    body = f"[bold {col}]{name}[/bold {col}]\n" + "  ".join(meta_parts)
    if props.get("description"):
        body += f"\n\n  [italic]{props['description'][:300]}[/italic]"
    if props_lines:
        body += "\n" + "\n".join(props_lines[:6])

    console.print()
    console.print(Panel(body, border_style=col, padding=(1, 2)))

    # Relationship tree (depth 2)
    tree = _build_relation_tree(graph, node_id, depth=2)
    console.print(tree)

    # List immediate neighbours for navigation
    neighbours: list[tuple[str, str]] = []
    for tgt in graph._graph.successors(node_id):
        n = graph._graph.nodes.get(tgt, {}).get("name", tgt)
        neighbours.append((tgt, n))
    for src in graph._graph.predecessors(node_id):
        if src not in {nid for nid, _ in neighbours}:
            n = graph._graph.nodes.get(src, {}).get("name", src)
            neighbours.append((src, n))

    if neighbours:
        console.print()
        console.print("  [dim]Navigate to:[/dim]")
        for i, (nid, nname) in enumerate(neighbours[:9], 1):
            netype = graph._graph.nodes.get(nid, {}).get("entity_type", "")
            nc = _color(netype)
            console.print(f"    [{nc}][{i}] {nname}[/{nc}] [dim]({netype})[/dim]")
        console.print("    [dim][b] back   [q] menu[/dim]")

        choice = _ask("Navigate [1-9] / [b]ack / [q]uit")
        if choice.isdigit() and 1 <= int(choice) <= len(neighbours):
            return neighbours[int(choice) - 1][0]
        return None  # back or quit

    return None


def _search_entities(graph: Any) -> str | None:
    """Simple name-based entity search.  Returns node_id to explore or None."""
    query = _ask_text("Search entities (name contains)")
    if not query:
        return None

    q = query.lower()
    matches = [
        (nid, data)
        for nid, data in graph._graph.nodes(data=True)
        if q in data.get("name", "").lower()
    ]
    matches.sort(key=lambda x: x[1].get("mention_count", 1), reverse=True)

    if not matches:
        console.print("  [dim]No matches.[/dim]")
        return None

    console.print()
    for i, (nid, data) in enumerate(matches[:10], 1):
        name = data.get("name", nid)
        etype = data.get("entity_type", "")
        col = _color(etype)
        degree = graph._graph.degree(nid)
        console.print(f"    [{i}] [{col}]{name}[/{col}] [dim]({etype}, {degree} links)[/dim]")

    choice = _ask("Explore [1-N] / [b]ack")
    if choice.isdigit() and 1 <= int(choice) <= len(matches):
        return matches[int(choice) - 1][0]
    return None


def _ask_text(prompt: str) -> str:
    try:
        return input(f"  {prompt}: ").strip()
    except (EOFError, KeyboardInterrupt):
        return ""


# ── main session ──────────────────────────────────────────────────────────────

def run_visualize_session(graph: Any) -> None:
    """Run an interactive graph exploration session."""
    stats = graph.get_stats()
    if stats["total_nodes"] == 0:
        console.print("[yellow]Knowledge graph is empty. Run /extract first.[/yellow]")
        return

    console.print()
    console.print(Panel(
        "[bold cyan]Knowledge Graph Explorer[/bold cyan]\n\n"
        f"  {stats['total_nodes']} entities · {stats['total_edges']} relationships\n\n"
        "[dim]  Navigate your knowledge graph interactively.[/dim]",
        border_style="cyan",
        padding=(1, 2),
    ))

    nav_stack: list[str] = []  # entity navigation stack

    while True:
        if nav_stack:
            # We're inside an entity — show it
            node_id = nav_stack[-1]
            next_id = _explore_entity(graph, node_id)
            if next_id:
                nav_stack.append(next_id)
            else:
                nav_stack.pop()
            continue

        # Main menu
        console.print()
        console.print("  [bold]Menu[/bold]")
        console.print("  [1] Graph overview")
        console.print("  [2] Browse entities by type")
        console.print("  [3] Explore entity (by number)")
        console.print("  [4] Search entities")
        console.print("  [q] Exit visualizer")

        choice = _ask("[1-4] / [q]uit")

        if choice == "q":
            break

        elif choice == "1":
            _show_overview(graph)

        elif choice == "2":
            console.print()
            console.print("  [bold]Choose type:[/bold]")
            for i, t in enumerate(_ENTITY_TYPES, 1):
                cnt = graph.get_stats().get("by_type", {}).get(t, 0)
                if cnt:
                    col = _color(t)
                    console.print(f"    [{i}] [{col}]{t}[/{col}] [dim]({cnt})[/dim]")
            console.print(f"    [{len(_ENTITY_TYPES) + 1}] All")

            tc = _ask("Type number / [b]ack")
            if tc.isdigit():
                idx = int(tc) - 1
                if idx == len(_ENTITY_TYPES):
                    entities = _show_entity_list(graph, None)
                elif 0 <= idx < len(_ENTITY_TYPES):
                    entities = _show_entity_list(graph, _ENTITY_TYPES[idx])
                else:
                    entities = []

                if entities:
                    ec = _ask("Explore entity number / [b]ack")
                    if ec.isdigit() and 1 <= int(ec) <= len(entities):
                        nav_stack.append(entities[int(ec) - 1][0])

        elif choice == "3":
            # Browse all then pick by number
            entities = _show_entity_list(graph, None)
            if entities:
                ec = _ask("Explore entity number / [b]ack")
                if ec.isdigit() and 1 <= int(ec) <= len(entities):
                    nav_stack.append(entities[int(ec) - 1][0])

        elif choice == "4":
            node_id = _search_entities(graph)
            if node_id:
                nav_stack.append(node_id)

    console.print("[dim]  Visualizer closed.[/dim]\n")



from __future__ import annotations

import logging
from datetime import datetime, timedelta, timezone
from typing import Any, AsyncIterator

from mneia.core.connector import (
    BaseConnector,
    ConnectorManifest,
    ConnectorMode,
    RawDocument,
)

logger = logging.getLogger(__name__)


class GoogleCalendarConnector(BaseConnector):
    manifest = ConnectorManifest(
        name="google-calendar",
        display_name="Google Calendar",
        version="0.1.0",
        description="Read events from Google Calendar (read-only)",
        author="mneia-team",
        mode=ConnectorMode.POLL,
        auth_type="oauth2",
        scopes=["https://www.googleapis.com/auth/calendar.readonly"],
        poll_interval_seconds=300,
        required_config=[],
        optional_config=[
            "google_client_id", "google_client_secret",
            "calendar_ids", "lookback_days",
        ],
    )

    def __init__(self) -> None:
        self._service: Any = None
        self._calendar_ids: list[str] = ["primary"]
        self._lookback_days: int = 30

    async def authenticate(self, config: dict[str, Any]) -> bool:
        self.last_error = ""
        try:
            from mneia.connectors.google_auth import (
                build_service,
                get_google_credentials,
            )

            client_id = config.get("google_client_id", "")
            client_secret = config.get("google_client_secret", "")
            account = config.get("account_name", "")

            creds = get_google_credentials(
                "calendar", client_id, client_secret, account=account,
            )
            self._service = build_service("calendar", "v3", creds)
            self._account = account

            cal_ids = config.get("calendar_ids", "")
            if cal_ids:
                self._calendar_ids = [
                    c.strip() for c in cal_ids.split(",") if c.strip()
                ]

            lookback = config.get("lookback_days", "")
            if lookback:
                self._lookback_days = int(lookback)

            return True
        except ImportError:
            self.last_error = "Google libraries not installed. Reinstall mneia."
            return False
        except Exception as e:
            self.last_error = f"Google Calendar auth failed: {e}"
            return False

    async def fetch_since(self, since: datetime | None) -> AsyncIterator[RawDocument]:
        if not self._service:
            return

        time_min = since or (datetime.now(timezone.utc) - timedelta(days=self._lookback_days))
        time_min_str = time_min.isoformat()

        for cal_id in self._calendar_ids:
            page_token = None
            while True:
                try:
                    result = (
                        self._service.events()
                        .list(
                            calendarId=cal_id,
                            timeMin=time_min_str,
                            maxResults=250,
                            singleEvents=True,
                            orderBy="startTime",
                            pageToken=page_token,
                        )
                        .execute()
                    )
                except Exception as e:
                    logger.error(f"Failed to fetch events from {cal_id}: {e}")
                    break

                for event in result.get("items", []):
                    doc = self._event_to_document(event, cal_id)
                    if doc:
                        yield doc

                page_token = result.get("nextPageToken")
                if not page_token:
                    break

    async def health_check(self) -> bool:
        if not self._service:
            return False
        try:
            self._service.calendarList().list(maxResults=1).execute()
            return True
        except Exception:
            return False

    def interactive_setup(self, account: str = "") -> dict[str, Any]:
        from mneia.connectors.google_auth import interactive_google_setup

        settings = interactive_google_setup("calendar", account=account)

        import typer

        cal_ids = typer.prompt(
            "  Calendar IDs (comma-separated, or 'primary' for default)",
            default="primary",
        )
        settings["calendar_ids"] = cal_ids

        lookback = typer.prompt("  Days of history to fetch", default="30")
        settings["lookback_days"] = lookback

        return settings

    def _event_to_document(self, event: dict[str, Any], cal_id: str) -> RawDocument | None:
        event_id = event.get("id", "")
        if not event_id:
            return None

        summary = event.get("summary", "Untitled Event")
        description = event.get("description", "")
        location = event.get("location", "")

        start_info = event.get("start", {})
        start_str = start_info.get("dateTime") or start_info.get("date", "")
        end_info = event.get("end", {})
        end_str = end_info.get("dateTime") or end_info.get("date", "")

        attendees = []
        for att in event.get("attendees", []):
            name = att.get("displayName") or att.get("email", "")
            if name:
                attendees.append(name)

        organizer = event.get("organizer", {})
        organizer_name = organizer.get("displayName") or organizer.get("email", "")

        content_parts = [f"# {summary}"]
        if start_str:
            content_parts.append(f"**When:** {start_str} — {end_str}")
        if location:
            content_parts.append(f"**Where:** {location}")
        if organizer_name:
            content_parts.append(f"**Organizer:** {organizer_name}")
        if attendees:
            content_parts.append(f"**Attendees:** {', '.join(attendees)}")
        if description:
            content_parts.append(f"\n{description}")

        content = "\n".join(content_parts)

        try:
            if "T" in start_str:
                timestamp = datetime.fromisoformat(start_str.replace("Z", "+00:00"))
            else:
                timestamp = datetime.fromisoformat(start_str)
        except (ValueError, TypeError):
            timestamp = datetime.now(timezone.utc)

        metadata: dict[str, Any] = {
            "calendar_id": cal_id,
            "event_status": event.get("status", ""),
            "event_type": event.get("eventType", "default"),
        }
        if event.get("recurringEventId"):
            metadata["recurring"] = True
        if event.get("hangoutLink"):
            metadata["meeting_link"] = event["hangoutLink"]
        if event.get("conferenceData"):
            for entry in event["conferenceData"].get("entryPoints", []):
                if entry.get("entryPointType") == "video":
                    metadata["meeting_link"] = entry.get("uri", "")
                    break

        source = f"google-calendar-{self._account}" if getattr(self, "_account", "") else "google-calendar"
        return RawDocument(
            source=source,
            source_id=event_id,
            content=content,
            content_type="event",
            title=summary,
            timestamp=timestamp,
            metadata=metadata,
            url=event.get("htmlLink"),
            participants=attendees,
        )



from __future__ import annotations

import logging
from typing import Any

from mneia.memory.graph import GraphEdge, KnowledgeGraph

logger = logging.getLogger(__name__)


def match_entities_by_name(
    entities_a: list[dict[str, Any]],
    entities_b: list[dict[str, Any]],
) -> list[tuple[str, str, float]]:
    matches = []
    for a in entities_a:
        a_name = a.get("name", "").lower().strip()
        for b in entities_b:
            b_name = b.get("name", "").lower().strip()
            if not a_name or not b_name:
                continue
            if a_name == b_name:
                matches.append((a.get("name", ""), b.get("name", ""), 1.0))
            elif a_name in b_name or b_name in a_name:
                matches.append((a.get("name", ""), b.get("name", ""), 0.7))
    return matches


def merge_duplicate_entities(graph: KnowledgeGraph) -> int:
    nodes = list(graph._graph.nodes(data=True))
    name_to_ids: dict[str, list[str]] = {}

    for node_id, data in nodes:
        name = data.get("name", "").lower().strip()
        if name:
            name_to_ids.setdefault(name, []).append(node_id)

    merged = 0
    for name, ids in name_to_ids.items():
        if len(ids) <= 1:
            continue
        canonical = ids[0]
        for dup_id in ids[1:]:
            for pred in list(graph._graph.predecessors(dup_id)):
                edge_data = graph._graph.edges[pred, dup_id]
                if not graph._graph.has_edge(pred, canonical):
                    graph.add_relationship(GraphEdge(
                        source_id=pred,
                        target_id=canonical,
                        relation=edge_data.get("relation", "related_to"),
                        weight=edge_data.get("weight", 1.0),
                        evidence=edge_data.get("evidence", ""),
                    ))
            for succ in list(graph._graph.successors(dup_id)):
                edge_data = graph._graph.edges[dup_id, succ]
                if not graph._graph.has_edge(canonical, succ):
                    graph.add_relationship(GraphEdge(
                        source_id=canonical,
                        target_id=succ,
                        relation=edge_data.get("relation", "related_to"),
                        weight=edge_data.get("weight", 1.0),
                        evidence=edge_data.get("evidence", ""),
                    ))
            graph.remove_node(dup_id)
            merged += 1

    logger.info(f"Merged {merged} duplicate entities")
    return merged



from __future__ import annotations

import json
import logging
import os
from pathlib import Path
from typing import Any

from pydantic import BaseModel, Field

logger = logging.getLogger(__name__)

MNEIA_DIR = Path.home() / ".mneia"

# Sentinel stored in config.json when a secret is held in the system keychain
_KEYCHAIN_SENTINEL = "__keychain__"

# LLM config fields that may be stored in the keychain
_SECRET_FIELDS = ("anthropic_api_key", "openai_api_key", "google_api_key")


def _get_secret(key: str, fallback: str = "") -> str:
    """Retrieve a secret from the system keychain; returns fallback if unavailable."""
    try:
        import keyring  # type: ignore[import-untyped]
        val = keyring.get_password("mneia", key)
        if val:
            return val
    except Exception:
        pass
    return fallback


def _set_secret(key: str, value: str) -> bool:
    """Store a secret in the system keychain. Returns True on success."""
    try:
        import keyring  # type: ignore[import-untyped]
        keyring.set_password("mneia", key, value)
        return True
    except Exception as exc:
        logger.debug("keyring unavailable (%s), storing plaintext", exc)
        return False
CONFIG_PATH = MNEIA_DIR / "config.json"
DATA_DIR = MNEIA_DIR / "data"
CONTEXT_DIR = MNEIA_DIR / "context"
TEMPLATES_DIR = MNEIA_DIR / "templates"
LOGS_DIR = MNEIA_DIR / "logs"
SOCKET_PATH = MNEIA_DIR / "mneia.sock"
PID_PATH = MNEIA_DIR / "daemon.pid"
STATS_DB_PATH = DATA_DIR / "agent_stats.db"


class LLMConfig(BaseModel):
    provider: str = "ollama"
    model: str = "phi3:mini"
    embedding_model: str = "nomic-embed-text"
    ollama_base_url: str = "http://localhost:11434"
    anthropic_api_key: str = ""
    openai_api_key: str = ""
    google_api_key: str = ""
    temperature: float = 0.1
    max_tokens: int = 2048


class ConnectorConfig(BaseModel):
    enabled: bool = False
    poll_interval_seconds: int = 300
    settings: dict[str, Any] = Field(default_factory=dict)
    last_checkpoint: str | None = None
    skipped_resources: list[str] = Field(default_factory=list)


class SafetyConfig(BaseModel):
    auto_approve_low_risk: bool = True
    approval_ttl_hours: int = 24
    blocked_operations: list[str] = Field(default_factory=list)


class MneiaConfig(BaseModel):
    llm: LLMConfig = Field(default_factory=LLMConfig)
    connectors: dict[str, ConnectorConfig] = Field(default_factory=dict)
    safety: SafetyConfig = Field(default_factory=SafetyConfig)
    context_output_dir: str = str(CONTEXT_DIR)
    auto_generate_context: bool = True
    context_regenerate_interval_minutes: int = 10
    context_min_changes_for_regen: int = 2
    enrichment_scrape_enabled: bool = False
    enrichment_max_scrape_pages: int = 5
    enrichment_scrape_delay_seconds: float = 2.0
    autonomous_enabled: bool = True
    autonomous_interval_minutes: int = 30
    autonomous_max_actions: int = 5
    autonomous_creativity_temperature: float = 0.7
    hermes_enabled: bool = True
    hermes_model: str = ""
    hermes_max_iterations: int = 10
    ner_enabled: bool = True
    reranker_enabled: bool = True
    graphrag_enabled: bool = False
    cognee_enabled: bool = False
    max_memory_mb: int = 2048
    log_level: str = "info"

    @classmethod
    def load(cls) -> MneiaConfig:
        if CONFIG_PATH.exists():
            data = json.loads(CONFIG_PATH.read_text(encoding="utf-8"))
            config = cls.model_validate(data)
        else:
            config = cls()
        config._resolve_secrets()
        config._apply_env_overrides()
        config._cleanup_dead_connectors()
        return config

    def _resolve_secrets(self) -> None:
        """Replace __keychain__ sentinels with real values from the system keychain."""
        for field in _SECRET_FIELDS:
            val = getattr(self.llm, field, "")
            if val == _KEYCHAIN_SENTINEL:
                resolved = _get_secret(field)
                if resolved:
                    self.llm.__dict__[field] = resolved
                else:
                    # Sentinel present but keychain empty — clear rather than leak sentinel
                    self.llm.__dict__[field] = ""

    def _cleanup_dead_connectors(self) -> None:
        from mneia.connectors import get_connector_manifest

        dead = [
            name for name in self.connectors
            if get_connector_manifest(name) is None
        ]
        if dead:
            for name in dead:
                del self.connectors[name]
            self.save()

    def _apply_env_overrides(self) -> None:
        env_mappings: dict[str, str] = {
            "MNEIA_LLM_PROVIDER": "llm.provider",
            "MNEIA_LLM_MODEL": "llm.model",
            "MNEIA_LLM_EMBEDDING_MODEL": "llm.embedding_model",
            "MNEIA_OLLAMA_BASE_URL": "llm.ollama_base_url",
            "MNEIA_ANTHROPIC_API_KEY": "llm.anthropic_api_key",
            "MNEIA_OPENAI_API_KEY": "llm.openai_api_key",
            "MNEIA_GOOGLE_API_KEY": "llm.google_api_key",
            "MNEIA_LOG_LEVEL": "log_level",
            "MNEIA_AUTONOMOUS_ENABLED": "autonomous_enabled",
        }
        for env_key, config_path in env_mappings.items():
            val = os.environ.get(env_key)
            if val is not None:
                self._set_value_no_save(config_path, val)

    def _set_value_no_save(self, key: str, value: str) -> None:
        parts = key.split(".")
        obj: Any = self
        for part in parts[:-1]:
            obj = getattr(obj, part)
        final_key = parts[-1]
        current = getattr(obj, final_key)
        if isinstance(current, bool):
            obj.__dict__[final_key] = value.lower() in ("true", "1", "yes")
        elif isinstance(current, int):
            obj.__dict__[final_key] = int(value)
        elif isinstance(current, float):
            obj.__dict__[final_key] = float(value)
        else:
            obj.__dict__[final_key] = value

    def save(self) -> None:
        MNEIA_DIR.mkdir(parents=True, exist_ok=True)
        CONFIG_PATH.write_text(
            self.model_dump_json(indent=2),
            encoding="utf-8",
        )

    def set_value(self, key: str, value: str) -> None:
        parts = key.split(".")
        obj: Any = self
        for part in parts[:-1]:
            if isinstance(obj, dict):
                obj = obj[part]
            else:
                obj = getattr(obj, part)
        final_key = parts[-1]
        if isinstance(obj, dict):
            obj[final_key] = value
        else:
            current = getattr(obj, final_key)
            if isinstance(current, bool):
                obj.__dict__[final_key] = value.lower() in ("true", "1", "yes")
            elif isinstance(current, int):
                obj.__dict__[final_key] = int(value)
            elif isinstance(current, float):
                obj.__dict__[final_key] = float(value)
            else:
                obj.__dict__[final_key] = value
        self.save()

    def get_value(self, key: str) -> Any:
        parts = key.split(".")
        obj: Any = self
        for part in parts:
            if isinstance(obj, dict):
                obj = obj[part]
            else:
                obj = getattr(obj, part)
        return obj


def ensure_dirs() -> None:
    for d in [MNEIA_DIR, DATA_DIR, CONTEXT_DIR, TEMPLATES_DIR, LOGS_DIR]:
        d.mkdir(parents=True, exist_ok=True)



from __future__ import annotations

import json
import logging
from dataclasses import dataclass, field
from enum import Enum

from mneia.core.llm import LLMClient
from mneia.memory.graph import KnowledgeGraph
from mneia.memory.store import MemoryStore

logger = logging.getLogger(__name__)


class ActionType(str, Enum):
    ENRICH = "enrich"
    CONNECT = "connect"
    INSIGHT = "insight"


@dataclass
class ReasoningAction:
    action_type: ActionType
    target: str
    description: str
    confidence: float = 0.5
    params: dict[str, str] = field(default_factory=dict)


@dataclass
class ReasoningPlan:
    actions: list[ReasoningAction] = field(default_factory=list)
    reasoning: str = ""


class ReasoningEngine:
    def __init__(
        self,
        llm: LLMClient,
        graph: KnowledgeGraph,
        store: MemoryStore,
        confidence_threshold: float = 0.6,
    ) -> None:
        self._llm = llm
        self._graph = graph
        self._store = store
        self._confidence_threshold = confidence_threshold

    async def analyze(self, max_actions: int = 5) -> ReasoningPlan:
        context = self._build_context()
        prompt = self._build_prompt(context, max_actions)

        try:
            response = await self._llm.generate(prompt)
            plan = self._parse_response(response)
            plan.actions = [
                a
                for a in plan.actions
                if a.confidence >= self._confidence_threshold
            ][:max_actions]
            return plan
        except Exception:
            logger.exception("Reasoning analysis failed")
            return ReasoningPlan(reasoning="Analysis failed due to LLM error")

    def _build_context(self) -> dict[str, object]:
        graph_stats = self._graph.get_stats()

        sparse_entities = []
        for nid, data in self._graph._graph.nodes(data=True):
            props = data.get("properties", {})
            desc = props.get("description", "")
            degree = self._graph._graph.degree(nid)
            if (not desc or len(desc) < 30) and degree < 2:
                sparse_entities.append({
                    "id": nid,
                    "name": data.get("name", ""),
                    "type": data.get("entity_type", ""),
                    "connections": degree,
                })
            if len(sparse_entities) >= 10:
                break

        isolated_nodes = [
            {
                "id": nid,
                "name": data.get("name", ""),
                "type": data.get("entity_type", ""),
            }
            for nid, data in self._graph._graph.nodes(data=True)
            if self._graph._graph.degree(nid) == 0
        ][:10]

        return {
            "graph_stats": graph_stats,
            "sparse_entities": sparse_entities,
            "isolated_nodes": isolated_nodes,
        }

    def _build_prompt(
        self, context: dict[str, object], max_actions: int,
    ) -> str:
        return (
            "You are a knowledge graph analyst. Analyze the following "
            "knowledge state and propose actions to improve it.\n\n"
            f"Graph statistics: {json.dumps(context['graph_stats'])}\n\n"
            f"Sparse entities (few connections, lacking descriptions):\n"
            f"{json.dumps(context['sparse_entities'], indent=2)}\n\n"
            f"Isolated nodes (no connections):\n"
            f"{json.dumps(context['isolated_nodes'], indent=2)}\n\n"
            f"Propose up to {max_actions} actions. For each action, "
            "respond with a JSON array of objects with these fields:\n"
            '- "action_type": one of "enrich", "connect", "insight"\n'
            '- "target": entity ID or topic\n'
            '- "description": what to do and why\n'
            '- "confidence": 0.0-1.0 how confident this is useful\n'
            '- "params": optional dict with extra parameters\n\n'
            "Action types:\n"
            '- "enrich": look up more info about a sparse entity\n'
            '- "connect": propose a relationship between entities\n'
            '- "insight": generate a synthetic insight document\n\n'
            "Respond with ONLY a JSON object like:\n"
            '{"reasoning": "...", "actions": [...]}'
        )

    def _parse_response(self, response: str) -> ReasoningPlan:
        cleaned = response.strip()
        if cleaned.startswith("```"):
            lines = cleaned.split("\n")
            lines = lines[1:]
            if lines and lines[-1].strip() == "```":
                lines = lines[:-1]
            cleaned = "\n".join(lines)

        try:
            data = json.loads(cleaned)
        except json.JSONDecodeError:
            logger.warning("Could not parse reasoning response as JSON")
            return ReasoningPlan(reasoning=response)

        actions = []
        for item in data.get("actions", []):
            try:
                action = ReasoningAction(
                    action_type=ActionType(item["action_type"]),
                    target=item["target"],
                    description=item.get("description", ""),
                    confidence=float(item.get("confidence", 0.5)),
                    params=item.get("params", {}),
                )
                actions.append(action)
            except (KeyError, ValueError):
                logger.debug(f"Skipping malformed action: {item}")

        return ReasoningPlan(
            actions=actions,
            reasoning=data.get("reasoning", ""),
        )
